'３次元流れ解析V128の標準ﾓｼﾞｭｰﾙ      ［作成］黒田英夫(2023/03)
Imports System.Drawing.Printing

Module FL3DCom
    '六面体２次要素または四面体２次要素を使用
    '自動メッシュ生成は「ＦＥＭ立体データ」ソフトによる
    '要素の節点番号の入力は、指定順序の反時計回り
    '0に固定する速度・圧力変数は除外してマトリックスを縮小
    '等高線図はカラーフィル、グラデーション機能有り
    '各作画画面においてマウスによる各種操作が可能

    'V116での改良点・・・入力データのファイル印刷メニューを廃止
    '                    （文字コードによる不具合のため）、
    '                    粘度式有りのニュートン流体対応の改善
    'V117での改良点・・・構造図で温度固定設定を内部点まで拡張（流入口の安定化のため）、
    '                    非定常解析で計算時間間隔の初期値のデフォールト表示を変更
    'V118での改良点・・・計算時の節点付番の改良で各座標軸方向のバンド幅を算出
    '                    （これまでは最長方向の座標軸を採用）
    'V119での改良点・・・構造図画面で「隠面処理なし」の選択を廃止、
    '                    構造図画面で条件設定部の表示を２区分から３区分に改善
    'V120での改良点・・・反復法で流出入量計算のバグを修正、
    '                    反復法の定常解析で手法改良（対角優先化）
    'V121での改良点・・・定常解析でバンドマトリックス法とスカイライン法の収束を改善
    'V122での改良点・・・反復法のバンド幅計算の修正
    'V123での改良点・・・UTF-8で非粘性の場合に速度ベクトル図が誤表示されるバグを修正、
    '                    速度ベクトル図で中間節点の有無を選択できるよう改善
    'V124での改良点・・・粘性有りの場合にスカイライン法を廃止
    '                    （バンドマトリックス法より計算時間がかかるため）、
    '                    粘性有りのバンドマトリックス法に非定常計算を追加、
    '                    粘性有りのバンドマトリックス法定常計算で安定化τpの
    '                    追加係数にAI利用
    'V125での改良点・・・四面体要素の廃止
    'V126での改良点・・・解析データのデータチェックに立体要素共通面の節点を追加
    'V127での改良点・・・実行形式は.exeと.exe.configを同一フォルダに配置
    'V128での改良点・・・フリーソフト化、VB2022に移行、文字コードはUTF-8のみ採用

    Public Const Pai As Single = 3.141593  'π

    Public CalcRes$         '計算結果表示データ
    Public Cr$              'CR+LFのコード
    Public DirOpen$         '立ち上げ時のディレクトリー名
    Public Fadd$            '連結するファイル名
    Public Fm$              '入力データのファイル名
    Public Fy$              '計算結果のファイル名
    Public Title$           'タイトル

    Public Fpara$(2)                'ﾊﾟﾗﾒｰﾀｰﾌｧｲﾙ名（ﾊﾞﾝﾄﾞﾏﾄﾘｯｸｽ法）

    Public Timst As Date            '計算開始時刻

    Public IncVisc As Integer       '粘性の有無（0:なし、1:有り）
    Public InitDataT As Integer     '温度の初期値データの有無（0:なし、1:有り）
    Public InitDataV As Integer     '速度・圧力の初期値データの有無（0:なし、1:有り）
    Public InOut As Integer         'ファイル入出力の種類（0:入力、1:出力）

    Public KAICalc As Integer       'AICalc実行の有無（0:なし、1:有り）

    Public KBodyForce As Integer    '物体力の有無（0:なし、1:有り）
    Public Kcolorfill As Integer    '等高線図のカラーフィルの有無（0:なし、1:有り）
    Public KcomSide As Integer      '共通辺の表示有無（0:なし、1:有り）
    Public KcontourPoint As Integer '等高線図で場所･数値表示の有無（0:なし、1:有り）
    Public Kconv As Integer         '対流項の有無（0:なし、1:有り）
    Public KdataChange As Integer   'データ変更の有無（0:なし、1:有り）
    Public KdispENum As Integer     '要素番号の表示有無（0:なし、1:有り）
    Public KdispPNum As Integer     '節点番号の表示有無（0:なし、1:有り）
    Public KdispSNum As Integer     '面番号の表示有無（0:なし、1:有り）
    Public KDrawDatMod As Integer   '構造図でのﾃﾞｰﾀ修正区分（0:個別、1:一括）
    Public KE1 As Integer           '開始要素番号
    Public KE2 As Integer           '最終要素番号
    Public KfileOK As Integer       'ﾌｧｲﾙのOK有無（0:なし、1:有り）
    Public KindBoundS As Integer    '一括設定の境界種別（0:速度･圧力固定、1:温度固定）
    Public KindElem As Integer      '要素種類（0:六面体20節点、1:四面体10節点）
    '    Public KindEncod As Integer     'ファイルの文字コード（0:UTF-8、1:Shif-JIS）
    Public KindFlow As Integer      '流れ種類（0:ﾆｭｰﾄﾝ流、1:非ﾆｭｰﾄﾝ流）
    Public KindPost As Integer      '後処理の種類（0:ﾍﾞｸﾄﾙ図、1:等高線図、2:ﾍﾞｸﾄﾙ図ｽﾗｲﾄﾞｼｮｰ、
    '                                3:等高線図ｽﾗｲﾄﾞｼｮｰ、4:端数処理、5:主要値表示、
    '                                6:外部領域への作用力計算）
    Public KindRelax As Integer     '粘度の種別（0:高粘度、1:低粘度）
    Public KindSolver As Integer    'ｿﾙﾊﾞｰ種類（0:反復法、1:ﾊﾞﾝﾄﾞﾏﾄﾘｯｸｽ法、2:ｽｶｲﾗｲﾝ法）
    Public KindTherm As Integer     '温度変化の種別（0:等温、1:非等温）
    Public KindWall As Integer      '壁面の区分（0:自動設定、1:ユーザー指定）
    Public KmouseDchange As Integer 'ﾏｳｽ指定要素のﾃﾞｰﾀ変更有無
    Public KmouseDset As Integer    'ﾏｳｽ指定要素のﾃﾞｰﾀ項目（1:速度・圧力固定節点、2:速度・圧力固定面、
    '                                3:温度固定節点、4:温度固定面、5:熱流束）
    Public KmouseElem As Integer    'ﾏｳｽ指定の要素番号
    Public KmousePoint As Integer   'ﾏｳｽ指定の節点番号
    Public KmouseSurf As Integer    'ﾏｳｽ指定の面番号
    Public KnewUFileA As Integer    '最近使用ﾌｧｲﾙAの更新有無
    Public KnumDisp As Integer      '構造作画での番号表示（0:なし、1:有り）
    Public KnumSb As Integer        'ﾌｫﾝﾄ文字ｻｲｽﾞの操作変数
    Public KnumSize As Integer      '構造作画での番号ｻｲｽﾞ
    Public KpressP As Integer       '基準圧力の節点番号（非粘性）
    Public KrearTreat As Integer    '隠面処理の有無（0:なし、1:有り）
    Public Kreverse As Integer      '逆計算の有無（0:なし、1:有り）
    Public Kscreen As Integer       '作画画面サイズ（0：全画面、1：通常画面）
    Public KStage As Integer        'ﾃﾞｰﾀ入力の段階番号
    Public Ktime As Integer         '出力メッシュ番号
    Public Kvalue As Integer        '作画の変数ｲﾝﾃﾞｯｸｽ
    Public KVecStep As Integer      '速度ﾍﾞｸﾄﾙ図で要素内の節点間隔（1:全節点、2:中間節点除外）
    Public Kvisc As Integer         '粘度式の種類（0:なし、1:べき乗、
    '                                2:常用対数、3:自然対数、4:温度2次式）
    Public Mstat As Integer         '解析種別（０:定常、１:非定常）
    Public Nall As Integer          '変数の総数
    Public NboundP, NboundPb As Integer       '速度・圧力固定の節点数
    Public NboundS, NboundSb As Integer       '速度・圧力固定の面数
    Public NBtempP, NBtempPb As Integer       '温度固定の節点数
    Public NBtempS, NBtempSb As Integer       '温度固定の面数
    Public Ndel As Integer          '削除要素数
    Public Ndim As Integer          '１節点当りの変数の数(=4、温度を除く)
    Public NDisp As Integer         '等高線の分割数
    Public NDmatrix As Integer      'FEM行列分割数
    Public Nelem As Integer         '要素数
    Public Nflux, Nfluxb As Integer '流束境界の面の数
    Public Ngaus As Integer         'ガウス積分点数(=3)
    Public NKmax As Integer         '最大繰り返し計算回数

    Public NparaSet As Integer      'パラメーター評価組数

    Public Npoint As Integer        '節点数
    Public NSdraw As Integer        '作画面の数（隠面処理時）
    Public Ntime As Integer         '非定常の出力時間メッシュ数

    Public Cfix As Single           'XY方向ﾋﾟｸｾﾙ差の補正係数
    Public Cratio As Single         '安定化τpの追加係数（基準0.005）
    Public Ctherm As Single         '比熱
    Public DistFactor As Single     '視点とモデルとの距離倍率（標準1）
    Public Dtime As Single          '非定常の出力の計算時間間隔
    Public DtimeInit As Single      '非定常の計算時間間隔の初期値
    Public DvelMax As Single        '非定常の計算1回の速度変化許容値
    Public EpsCalc As Single        '収束基準誤差率
    Public EyeAngle As Single       '視点と原点を結ぶ線の水平面との角度
    Public Eyeh As Single           '視点高さ
    Public HFactor As Single        '視点の高さ倍率（基準1）
    Public PressStd As Single       '基準圧力（非粘性）
    Public Rlenmax As Single        'ﾓﾃﾞﾙの1軸の最大長さ
    Public Rlenmodel As Single      '視点の距離（座標系間）
    Public Rou As Single            '密度
    Public Rmyu As Single           '粘度
    Public Rlamda As Single         '熱伝導率
    Public Tempi As Single          '一定温度または初期温度
    Public TempWall As Single       '壁面温度（一括指定）
    Public TimeMax As Single        '非定常の打切り時間
    Public TimeNew As Single        '現在の計算時刻
    Public VdispMax As Single       '作画変数の最高値
    Public VdispMin As Single       '作画変数の最低値
    Public VecMaxb As Single        '最大速度
    Public VelAve As Single         '入口平均流速絶対値（逆解析用）
    Public XaxisMax As Single       'Ｘ軸の最大値
    Public XaxisMin As Single       'Ｘ軸の最小値
    Public Xlower As Single         '作画領域のX最小値
    Public XRotate As Single        'モデルのX軸回りの回転角（右ねじ正、°）
    Public Xupper As Single         '作画領域のX最大値
    Public XYscale As Single        '作画スケール
    Public YaxisMax As Single       'Ｙ軸の最大値
    Public YaxisMin As Single       'Ｙ軸の最小値
    Public Ylower As Single         '作画領域のY最小値
    Public YRotate As Single        'モデルのY軸回りの回転角（右ねじ正、°）
    Public Yupper As Single         '作画領域のY最大値
    Public ZRotate As Single        'モデルのZ軸回りの回転角（右ねじ正、°）

    Public UFileA(6) As String      '最近使用したﾃﾞｰﾀﾌｧｲﾙの保存配列

    Public JfixP(,) As Integer      '節点の速度・圧力固定の有無(0:固定なし、1または-1:固定有り)
    Public JfixS(,) As Integer      '面の速度・圧力固定の有無(0:固定なし、1:固定有り)
    Public KAppPoint() As Integer   'ﾃﾞｰﾀ設定の節点
    Public KAppSide(,) As Integer   'ﾃﾞｰﾀ設定の要素指定面
    Public Kcomp() As Integer       '元ﾃﾞｰﾀ節点の計算変数順序
    Public Kdel() As Integer        '削除要素番号
    Public KdrawElem() As Integer   '各要素の作画有無（0:なし、1:有り）
    Public KE3DCut(2) As Integer    '構造図の面切断の指定（座標軸番号と正負選択）
    Public KEInside(,) As Integer   '全体構造での要素各面の内部区分（0:外周、1:内部）
    Public KEorder() As Integer     '作画順の要素番号
    Public KinnerLine() As Integer  '内部辺の指定（0:なし、1:有り）
    Public Koption(3) As Integer    'オプション有無（1:バックアップ、
    '                                2:要素入力画面の表示、3:節点入力画面の表示）
    Public KRcomp() As Integer      '計算順序節点の元ﾃﾞｰﾀ節点順序
    Public KSelem(,) As Integer     '各要素面の隠面指定（0:なし、1：有り、-1:視線外）
    Public KSorder() As Integer     '作画順の要素の面番号
    Public KsurfFlux(,) As Integer  '流束境界指定の要素・面番号
    Public KsurfPoint(8) As Integer '面の8節点の節点順位
    Public KsurfRest(,) As Integer  '速度・圧力固定の要素・面番号
    Public KsurfTrest(,) As Integer '温度固定の要素・面番号
    Public Lcomp() As Integer       '全変数の縮小ﾏﾄﾘｯｸｽ変数番号
    Public NNelem(,) As Integer     '各要素における全節点番号
    Public NPrefer() As Integer     '要素計算における節点変数の対応番号
    Public NrestP() As Integer      '速度・圧力固定の節点番号
    Public NTrestP() As Integer     '温度固定の節点番号
    Public Nw() As Integer          '各節点の共通要素数

    Public Accel(3) As Single       '物体力の加速度
    Public Avisc(5) As Single       '粘度式の係数
    Public AxisMax(3) As Single     'モデルのXYZ各軸の最大値
    Public AxisMin(3) As Single     'モデルのXYZ各軸の最小値
    Public BoundSVal(,) As Single   '境界一括設定の変数値
    Public CenterPos(3) As Single   '構造中心の座標
    Public Ds(20), Dt(20), Du(20) As Double  '形状関数のξ偏微分、η偏微分、ζ偏微分
    Public Dss(20), Dtt(20), Duu(20) As Double  '形状関数の2階のstu偏微分
    Public Dst(20), Dtu(20), Dus(20) As Double
    Public Dx(20), Dy(20), Dz(20) As Double  '形状関数のx偏微分、y偏微分、z偏微分

    Public ParaSet(,) As Single     'パラメーターセット

    Public PDVel(,) As Double       '節点での速度の偏微分
    Public PDVelX(,) As Double      '節点での速度の偏微分(前回)
    Public PDVelXX(,) As Double     '節点での速度の偏微分(前々回)
    Public Pnorm() As Single        '垂直流束
    Public PosGaus(3) As Single     '積分点の位置
    Public RinvJ(3, 3) As Double    '逆ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽ（分母割算省略形）
    Public RinvJT(3, 3) As Double   '逆ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽ（正規）
    Public RJmat(3, 3) As Double    'ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽ
    Public Shape(20) As Double      '形状関数
    Public TE(20) As Double         '要素節点での値
    Public TempFixP() As Single     '節点の固定温度
    Public TempFixS() As Single     '面の固定温度
    Public TempInit() As Single     '温度の初期値
    Public Tff() As Double          '流れ計算の右辺ﾍﾞｸﾄﾙ
    Public Tffx() As Double         '前時間ｽﾃｯﾌﾟの速度・圧力
    Public Tfp() As Single          'FEM計算の解
    Public TG(27) As Double         'ｶﾞｳｽ点での値
    Public Tr(20, 27) As Single     'ｶﾞｳｽ点・節点の変換ﾏﾄﾘｯｸｽ
    Public TransMat(4, 4) As Single 'モデルの合成アフィン変換行列
    Public UE(20), VE(20), WE(20) As Double   '要素節点の速度U、V、W
    Public UEb(20), VEb(20), WEb(20) As Double  '要素節点の前回速度U、V、W
    Public UG(27), VG(27), WG(27) As Double   'ｶﾞｳｽ点の速度U、V、W
    Public VE3DCut(2) As Single     '構造図の面切断の座標値と許容誤差
    Public VfixP(,) As Single       '節点の速度・圧力の固定値
    Public VfixS(,) As Single       '面の速度・圧力の固定値
    Public Vinit(,) As Single       '速度・圧力の初期値
    Public VolE() As Single         '要素体積
    Public WeightG(3) As Single     '積分点の重み
    Public Xdata(), Ydata(), Zdata() As Single    '節点座標
    Public Xdraw(8), Ydraw(8) As Single   '色塗り多角形の座標
    Public XE(20), YE(20), ZE(20) As Single  '要素計算の配列
    Public XG(27), YG(27), ZG(27) As Double  'ｶﾞｳｽ点の座標
    Public Xmodel(), Ymodel(), Zmodel() As Single 'モデル座標における節点座標
    Public Xmouse(2), Ymouse(2) As Single   'ﾏｳｽ位置座標
    Public Xscreen(), Yscreen() As Single   'スクリーンにおける節点座標
    Public Xview(), Yview(), Zview() As Single  '視点座標における節点座標（左手系）

    Sub AreaCalc(ByRef area As Single, ByVal x1 As Single,
    ByVal y1 As Single, ByVal x2 As Single, ByVal y2 As Single,
    ByVal xb As Single, ByVal yb As Single)
        '三角形の符号付き面積算出（最初の2点に対して左側は正、右側は負）
        'area･･･符号付き面積、三角形の2倍
        'x1,y1,x2,y2･･･2点の座標
        'xb,yb･･･対象点の座標
        area = x1 * y2 - y1 * x2 + x2 * yb - y2 * xb + xb * y1 - yb * x1
    End Sub

    Sub AreaCheck(ByRef kh As Integer, ByVal nmax As Integer, ByVal epsA As Single,
        ByRef xbuf() As Single, ByRef xbuf2() As Single, ByRef ybuf() As Single,
        ByRef ybuf2() As Single)
        '2面の重なりﾁｪｯｸ
        'kh･･･区分（0:関係なし、1:重なり有り）
        'nmax･･･面の節点数
        'epsA･･･面積誤差の基準
        'xbuf(),ybuf(),xbuf2(),ybuf2()･･･2面の節点座標
        Dim ic As Integer
        Dim area, x1, x2, xb, y1, y2, yb As Single
        Dim i, j As Integer

        kh = 0
        For i = 1 To nmax
            xb = xbuf(i)
            yb = ybuf(i)
            x1 = xbuf2(nmax)
            y1 = ybuf2(nmax)
            ic = 0
            For j = 1 To nmax - 1
                x2 = xbuf2(j)
                y2 = ybuf2(j)
                Call AreaCalc(area, x1, y1, x2, y2, xb, yb) '三角形の符号付き面積算出
                If area < epsA Then
                    ic = 1
                    Exit For
                Else
                    x1 = x2
                    y1 = y2
                End If
            Next j
            If ic = 0 Then
                kh = 1
                Exit For
            End If
        Next i

    End Sub

    Sub AxisCheck(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef rlenmax As Single)
        'X、Y、Z軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'rlenmax･･･1軸の最大長さ
        Dim ie, ii, ik, j, k, nmax As Integer
        Dim b1 As Single
        If KindElem = 0 Then
            nmax = 20
        End If
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        AxisMin(1) = Xdata(k)
                        AxisMin(2) = Ydata(k)
                        AxisMin(3) = Zdata(k)
                        For ii = 1 To 3
                            AxisMax(ii) = AxisMin(ii)
                        Next ii
                    Else
                        For ii = 1 To 3
                            Select Case ii
                                Case 1
                                    b1 = Xdata(k)
                                Case 2
                                    b1 = Ydata(k)
                                Case 3
                                    b1 = Zdata(k)
                            End Select
                            If b1 < AxisMin(ii) Then
                                AxisMin(ii) = b1
                            ElseIf b1 > AxisMax(ii) Then
                                AxisMax(ii) = b1
                            End If
                        Next ii
                    End If
                Next j
            End If
        Next ie

        rlenmax = AxisMax(1) - AxisMin(1)
        For ii = 2 To 3
            b1 = AxisMax(ii) - AxisMin(ii)
            If b1 > rlenmax Then
                rlenmax = b1
            End If
        Next ii

    End Sub

    Sub AxisCheck2(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef xmin As Single, ByRef xmax As Single, _
    ByRef ymin As Single, ByRef ymax As Single)
        'ｽｸﾘｰﾝにおけるX、Y軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim ie, ik, k, j, nmax As Integer
        If KindElem = 0 Then
            nmax = 20
        End If
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        xmin = Xscreen(k) : xmax = xmin
                        ymin = Yscreen(k) : ymax = ymin
                    Else
                        If Xscreen(k) < xmin Then
                            xmin = Xscreen(k)
                        ElseIf Xscreen(k) > xmax Then
                            xmax = Xscreen(k)
                        End If
                        If Yscreen(k) < ymin Then
                            ymin = Yscreen(k)
                        ElseIf Yscreen(k) > ymax Then
                            ymax = Yscreen(k)
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub AxisCheckM(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef rlenmax As Single, ByRef axmax() As Single, ByRef axmin() As Single)
        'ﾓﾃﾞﾙ座標のX、Y、Z軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'rlenmax･･･1軸の最大長さ
        'axmax()･･･X,Y,Z座標の最大値
        'axmin()･･･X,Y,Z座標の最小値
        Dim ie, ii, ik, j, k, nmax As Integer
        Dim b1 As Single

        If KindElem = 0 Then
            nmax = 20
        End If
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        axmin(1) = Xmodel(k)
                        axmin(2) = Ymodel(k)
                        axmin(3) = Zmodel(k)
                        For ii = 1 To 3
                            axmax(ii) = axmin(ii)
                        Next ii
                    Else
                        For ii = 1 To 3
                            Select Case ii
                                Case 1
                                    b1 = Xmodel(k)
                                Case 2
                                    b1 = Ymodel(k)
                                Case 3
                                    b1 = Zmodel(k)
                            End Select
                            If b1 < axmin(ii) Then
                                axmin(ii) = b1
                            ElseIf b1 > axmax(ii) Then
                                axmax(ii) = b1
                            End If
                        Next ii
                    End If
                Next j
            End If
        Next ie

        rlenmax = axmax(1) - axmin(1)
        For ii = 2 To 3
            b1 = axmax(ii) - axmin(ii)
            If b1 > rlenmax Then
                rlenmax = b1
            End If
        Next ii

    End Sub

    Sub AxisCheckS(ByRef axmax() As Single, ByRef axmin() As Single)
        'X、Y、Z軸の最小・最大値
        Dim i, j As Integer
        Dim buf(3) As Single

        For i = 1 To Npoint
            buf(1) = Xdata(i)
            buf(2) = Ydata(i)
            buf(3) = Zdata(i)
            If i = 1 Then
                For j = 1 To 3
                    axmax(j) = buf(j)
                    axmin(j) = buf(j)
                Next j
            Else
                For j = 1 To 3
                    If buf(j) > axmax(j) Then
                        axmax(j) = buf(j)
                    ElseIf buf(j) < axmin(j) Then
                        axmin(j) = buf(j)
                    End If
                Next j
            End If
        Next i

    End Sub

    Sub AxisCheckSet(ByVal np As Integer, ByRef xrange() As Single, ByRef yrange() As Single,
        ByRef xdb() As Single, ByRef ydb() As Single)
        'X･Y座標の最小・最大値のﾁｪｯｸ（ｾｯﾄ）
        'np･･･節点数
        'xrange()･･･X座標最小値･最大値
        'yrange()･･･Y座標最小値･最大値
        'xdb(),ydb()･･･節点座標
        Dim i As Integer

        For i = 1 To np
            If i = 1 Then
                xrange(1) = xdb(1)
                xrange(2) = xdb(1)
                yrange(1) = ydb(1)
                yrange(2) = ydb(1)
            Else
                If xdb(i) < xrange(1) Then
                    xrange(1) = xdb(i)
                ElseIf xdb(i) > xrange(2) Then
                    xrange(2) = xdb(i)
                End If
                If ydb(i) < yrange(1) Then
                    yrange(1) = ydb(i)
                ElseIf ydb(i) > yrange(2) Then
                    yrange(2) = ydb(i)
                End If
            End If
        Next i

    End Sub

    Sub AxisValueTreat(ByRef xbuf() As Single, ByRef ybuf() As Single, ByRef zbuf() As Single)
        '座標値の端数処理
        'xbuf(),ybuf(),zbuf()･･･節点の座標
        Dim i, ib, ie, kind, nb As Integer
        Dim b2, rlenb As Single
        ReDim KdrawElem(Nelem)

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie

        Call AxisCheck(1, Nelem, rlenb)
        'X、Y、Z軸の最小・最大値のﾁｪｯｸ

        b2 = rlenb * 0.00002
        nb = 0
        i = Int(b2)
        If i > 0 Then
            kind = 0
            Do
                ib = CInt(b2)
                If ib < 10 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 0.1
                End If
            Loop
        Else
            kind = 1
            Do
                ib = CInt(b2)
                If ib > 0 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 10
                End If
            Loop
        End If

        For i = 1 To Npoint
            xbuf(i) = Xdata(i)
            ybuf(i) = Ydata(i)
            zbuf(i) = Zdata(i)
        Next i
        For i = 1 To Npoint
            If Xdata(i) <> 0 Then
                Call RoundTreat(kind, nb, xbuf(i)) '丸め処理
            End If
            If Ydata(i) <> 0 Then
                Call RoundTreat(kind, nb, ybuf(i)) '丸め処理
            End If
            If Zdata(i) <> 0 Then
                Call RoundTreat(kind, nb, zbuf(i)) '丸め処理
            End If
        Next i

    End Sub

    Sub BandWidth2(ByRef mb As Integer, ByVal kcbuf() As Integer)
        'ﾊﾞﾝﾄﾞ幅の計算
        'mb･･･ﾊｰﾌﾊﾞﾝﾄﾞ幅
        'kcbuf()･･･元の節点の変数順序
        Dim i, ie, ii, j, jj, k, mx, nmax As Integer

        If KindElem = 0 Then
            nmax = 20
        End If
        mx = 0
        For ie = 1 To Nelem
            For ii = 1 To nmax - 1
                i = NNelem(ie, ii)
                For jj = ii + 1 To nmax
                    j = NNelem(ie, jj)
                    k = Math.Abs(kcbuf(i) - kcbuf(j))
                    If k > mx Then mx = k
                Next jj
            Next ii
        Next ie
        mb = Ndim * (mx + 1)

    End Sub

    Sub BDatDupCheck(ByRef nd As Integer)
        '速度・圧力固定節点の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim kbuf(NboundP) As Integer

        nd = 0
        For i = 1 To NboundP - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To NboundP
                    If NrestP(i) = NrestP(ii) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        For j = 1 To Ndim
                            If JfixP(ii, j) <> 0 Then
                                JfixP(i, j) = JfixP(ii, j)
                                VfixP(i, j) = VfixP(ii, j)
                            End If
                        Next j
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nobuf(NboundP), jfbuf(NboundP, Ndim) As Integer
            Dim vfbuf(NboundP, Ndim) As Single
            For i = 1 To NboundP
                nobuf(i) = NrestP(i)
                For j = 1 To Ndim
                    jfbuf(i, j) = JfixP(i, j)
                    vfbuf(i, j) = VfixP(i, j)
                Next j
            Next i
            nb = NboundP - nd
            ReDim NrestP(nb), JfixP(nb, Ndim), VfixP(nb, Ndim)
            nb = 0
            For i = 1 To NboundP
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    NrestP(nb) = nobuf(i)
                    For j = 1 To Ndim
                        JfixP(nb, j) = jfbuf(i, j)
                        VfixP(nb, j) = vfbuf(i, j)
                    Next j
                End If
            Next i
            NboundP = nb
            NboundPb = nb
        End If

    End Sub

    Sub BSDatDupCheck(ByRef nd As Integer)
        '速度・圧力固定面の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim ksbuf(NboundS) As Integer

        nd = 0
        For i = 1 To NboundS - 1
            If ksbuf(i) = 0 Then
                For ii = i + 1 To NboundS
                    If KsurfRest(i, 1) = KsurfRest(ii, 1) Then
                        If KsurfRest(i, 2) = KsurfRest(ii, 2) Then
                            nd = nd + 1
                            ksbuf(ii) = -1
                            For j = 1 To Ndim
                                If JfixS(ii, j) <> 0 Then
                                    JfixS(i, j) = JfixS(ii, j)
                                    VfixS(i, j) = VfixS(ii, j)
                                End If
                            Next j
                        End If
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim kbuf(NboundS, 2), jbuf(NboundS, Ndim) As Integer
            Dim buf(NboundS, Ndim) As Single
            For i = 1 To NboundS
                kbuf(i, 1) = KsurfRest(i, 1)
                kbuf(i, 2) = KsurfRest(i, 2)
                For j = 1 To Ndim
                    jbuf(i, j) = JfixS(i, j)
                    buf(i, j) = VfixS(i, j)
                Next j
            Next i
            nb = NboundS - nd
            ReDim KsurfRest(nb, 2)
            ReDim JfixS(nb, Ndim), VfixS(nb, Ndim)
            nb = 0
            For i = 1 To NboundS
                If ksbuf(i) = 0 Then
                    nb = nb + 1
                    KsurfRest(nb, 1) = kbuf(i, 1)
                    KsurfRest(nb, 2) = kbuf(i, 2)
                    For j = 1 To Ndim
                        JfixS(nb, j) = jbuf(i, j)
                        VfixS(nb, j) = buf(i, j)
                    Next j
                End If
            Next i
            NboundS = nb
            NboundSb = nb
        End If

    End Sub

    Sub CfileIn1U(ByVal fxx$)
        '計算結果ﾌｧｲﾙの情報読込み
        'UTF-8･･･BOMなし
        'fxx$･･･計算結果ﾌｧｲﾙ名
        Dim aa$
        Dim i As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        For i = 1 To 3
            aa$ = sr.ReadLine
        Next i
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) = "解析種類（0:定常、1:非定常）" Then
            Mstat = dat(1)
            If Mstat > 0 Then
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                TimeMax = dat(1)
                Ntime = dat(3)
            End If
            Do
                aa$ = sr.ReadLine
                If aa$ = "等温流" Then
                    KindTherm = 0
                    Exit Do
                ElseIf aa$ = "非等温流" Then
                    KindTherm = 1
                    Exit Do
                End If
            Loop
        Else
            Mstat = 0
        End If

        sr.Close()

    End Sub

    Sub CoordiChange(ByVal rlenb As Single)
        '節点座標の視点座標への変換
        'rlenb･･･ﾓﾃﾞﾙ座標原点と視点座標との距離
        Dim ip As Integer
        For ip = 1 To Npoint
            Xview(ip) = Xmodel(ip)
            Yview(ip) = Zmodel(ip)
            Zview(ip) = Ymodel(ip) + rlenb
        Next ip

    End Sub

    Sub DataCheck(ByVal kdisp As Integer, ByRef num As Integer)
        '入力ﾃﾞｰﾀのｴﾗｰﾁｪｯｸ
        'kdisp･･･エラー無しのメッセージの有無（0:なし、1:有り）
        'num･･･エラーの数（上限10）
        Dim aa$, aerr1$
        Dim ib, ic, ie, ip, j, jmax, k, kb, kmax, nb, nmax, nx As Integer
        Dim b1, b2, b3 As Single
        Dim abuf$(10)

        If KindElem = 0 Then
            kmax = 6
            nmax = 20
        End If
        aerr1$ = "になっています！"
        num = 0

        If Npoint < 20 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  節点数が不足です！"
        End If
        If Nelem < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  要素数が" + Str$(Nelem) + aerr1$
        End If
        If NboundP < 1 And NboundS < 1 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            If IncVisc = 0 Then
                abuf$(num) = aa$ + "  節点や面の速度ポテンシャル固定がありません！"
            Else
                abuf$(num) = aa$ + "  節点や面の速度・圧力固定がありません！"
            End If
        End If
        If Kvisc < 0 Or Kvisc > 4 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  粘度式の番号が" + Str$(Kvisc) + aerr1$
        End If

        '要素のﾁｪｯｸ
        For ie = 1 To Nelem
            For j = 1 To nmax
                nb = NNelem(ie, j)
                If nb < 1 Or nb > Npoint Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  要素番号" + Str$(ie) + "の"
                    aa$ = aa$ + " 節点番号" + Str$(nb)
                    abuf$(num) = aa$ + "は定義されていません！"
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
                If j < nmax Then
                    For k = j + 1 To nmax
                        nx = NNelem(ie, k)
                        If nb = nx Then
                            num = num + 1
                            aa$ = "Error" + LTrim$(Str$(num))
                            aa$ = aa$ + "  要素番号" + Str$(ie) + "で"
                            aa$ = aa$ + " ２つの節点番号が" + Str$(nb)
                            abuf$(num) = aa$ + aerr1$
                            If num = 10 Then
                                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                                Exit Sub
                            End If
                        End If
                    Next k
                End If
            Next j
        Next ie

        '節点のﾁｪｯｸ
        For ip = 1 To Npoint - 1
            For j = ip + 1 To Npoint
                b1 = Xdata(ip) - Xdata(j)
                b2 = Ydata(ip) - Ydata(j)
                b3 = Zdata(ip) - Zdata(j)
                If b1 = 0 And b2 = 0 And b3 = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  節点番号" + Str$(ip) + "、" + Str$(j)
                    abuf$(num) = aa$ + "が" + " 同じ座標位置" + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next ip

        '不要節点のﾁｪｯｸ
        Dim kbuf(Npoint) As Integer
        For ie = 1 To Nelem
            For j = 1 To 20
                kbuf(NNelem(ie, j)) = 1
            Next j
        Next ie
        For ip = 1 To Npoint
            If kbuf(ip) = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf(num) = aa$ + "  節点番号" + Str$(ip) + "が使われていません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
        Next ip

        '要素共通辺のﾁｪｯｸ
        ReDim kbuf(Npoint)
        For ie = 1 To Nelem
            For j = 1 To 7 Step 2
                ip = NNelem(ie, j)
                If kbuf(ip) = 0 Then
                    kbuf(ip) = 1    '頂点節点に指定
                End If
            Next j
            For j = 13 To 19 Step 2
                ip = NNelem(ie, j)
                If kbuf(ip) = 0 Then
                    kbuf(ip) = 1    '頂点節点に指定
                End If
            Next j
        Next ie
        For ie = 1 To Nelem
            For j = 2 To 8 Step 2
                ip = NNelem(ie, j)
                If kbuf(ip) > 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    abuf(num) = aa$ + "  節点番号" + Str$(ip) + "が頂点と中間節点に使われています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
            For j = 9 To 12
                ip = NNelem(ie, j)
                If kbuf(ip) > 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    abuf(num) = aa$ + "  節点番号" + Str$(ip) + "が頂点と中間節点に使われています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
            For j = 14 To 20 Step 2
                ip = NNelem(ie, j)
                If kbuf(ip) > 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    abuf(num) = aa$ + "  節点番号" + Str$(ip) + "が頂点と中間節点に使われています！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        Next ie

        If IncVisc = 0 Then
            If NboundP > 0 Then     '速度ﾎﾟﾃﾝｼｬﾙ固定の節点ﾃﾞｰﾀのﾁｪｯｸ
                For ib = 1 To NboundP
                    nb = NrestP(ib)
                    If nb < 1 Or nb > Npoint Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  速度ポテンシャル固定節点の固定番号" + Str$(ib)
                        aa$ = aa$ + "の" + " 節点番号" + Str$(nb)
                        abuf$(num) = aa$ + "は定義されていません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next ib
            End If

            If NboundS > 0 Then     '速度ﾎﾟﾃﾝｼｬﾙ固定の面ﾃﾞｰﾀのﾁｪｯｸ
                For ib = 1 To NboundS
                    kb = KsurfRest(ib, 1)
                    If kb < 1 Or kb > Nelem Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  速度ポテンシャル固定面の固定番号" + Str$(ib)
                        abuf$(num) = aa$ + "で　要素番号が" + Str$(kb) + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                    kb = KsurfRest(ib, 2)
                    If kb < 1 Or kb > kmax Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  速度ポテンシャル固定面の固定番号" + Str$(ib)
                        abuf$(num) = aa$ + "で 面番号が" + Str$(kb) + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next ib
            End If

        Else
            If NboundP > 0 Then     '速度・圧力固定の節点ﾃﾞｰﾀのﾁｪｯｸ
                For ib = 1 To NboundP
                    nb = NrestP(ib)
                    If nb < 1 Or nb > Npoint Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  速度・圧力固定節点の固定番号" + Str$(ib)
                        aa$ = aa$ + "の" + " 節点番号" + Str$(nb)
                        abuf$(num) = aa$ + "は定義されていません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                    ic = 0
                    For j = 1 To Ndim
                        If JfixP(ib, j) <> 0 Then
                            ic = 1
                            Exit For
                        End If
                    Next j
                    If ic = 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  速度・圧力固定節点の固定番号" + Str$(ib)
                        abuf$(num) = aa$ + "で固定がありません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next ib
            End If

            If NboundS > 0 Then     '速度・圧力固定の面ﾃﾞｰﾀのﾁｪｯｸ
                For ib = 1 To NboundS
                    kb = KsurfRest(ib, 1)
                    If kb < 1 Or kb > Nelem Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  速度・圧力固定面の固定番号" + Str$(ib)
                        abuf$(num) = aa$ + "で　要素番号が" + Str$(kb) + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                    kb = KsurfRest(ib, 2)
                    If kb < 1 Or kb > kmax Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  速度・圧力固定面の固定番号" + Str$(ib)
                        abuf$(num) = aa$ + "で 面番号が" + Str$(kb) + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                    ic = 0
                    For j = 1 To Ndim
                        If JfixS(ib, j) <> 0 Then
                            ic = 1
                            Exit For
                        End If
                    Next j
                    If ic = 0 Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  速度・圧力固定面の固定番号" + Str$(ib)
                        abuf$(num) = aa$ + "で 固定がありません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next ib
            End If

            If NBtempP > 0 Then     '温度固定の節点ﾃﾞｰﾀのﾁｪｯｸ
                For ib = 1 To NBtempP
                    nb = NTrestP(ib)
                    If nb < 1 Or nb > Npoint Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  温度固定節点の固定番号" + Str$(ib) + "の"
                        aa$ = aa$ + " 節点番号" + Str$(nb)
                        abuf$(num) = aa$ + "は定義されていません！"
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next ib
            End If

            If NBtempS > 0 Then     '温度固定の面ﾃﾞｰﾀのﾁｪｯｸ
                For ib = 1 To NBtempS
                    kb = KsurfTrest(ib, 1)
                    If kb < 1 Or kb > Nelem Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  温度固定面の固定番号" + Str$(ib) + "で"
                        abuf$(num) = aa$ + " 要素番号が" + Str$(kb) + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                    kb = KsurfTrest(ib, 2)
                    If kb < 1 Or kb > kmax Then
                        num = num + 1
                        aa$ = "Error" + LTrim$(Str$(num))
                        aa$ = aa$ + "  温度固定面の固定番号" + Str$(ib) + "で"
                        abuf$(num) = aa$ + " 面番号が" + Str$(kb) + aerr1$
                        If num = 10 Then
                            Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                            Exit Sub
                        End If
                    End If
                Next ib
            End If

        End If

        If Nflux > 0 Then   '流束ﾃﾞｰﾀのﾁｪｯｸ
            For ib = 1 To Nflux
                kb = KsurfFlux(ib, 1)
                If kb < 1 Or kb > Nelem Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    If IncVisc = 0 Then
                        aa$ = aa$ + "  面垂直流の番号" + Str$(ib)
                    Else
                        aa$ = aa$ + "  熱流束境界の番号" + Str$(ib)
                    End If
                    abuf$(num) = aa$ + "で　要素番号が" + Str$(kb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
                kb = KsurfFlux(ib, 2)
                If kb < 1 Or kb > kmax Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    If IncVisc = 0 Then
                        aa$ = aa$ + "  面垂直流の番号" + Str$(ib)
                    Else
                        aa$ = aa$ + "  熱流束境界の番号" + Str$(ib)
                    End If
                    abuf$(num) = aa$ + "で 面番号が" + Str$(kb) + aerr1$
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next ib
        End If

        If IncVisc = 0 Then
            If KpressP < 1 Or KpressP > Npoint Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                aa$ = aa$ + "  基準圧力の節点番号" + Str$(KpressP)
                abuf$(num) = aa$ + "は定義されていません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
        End If

        If Rou = 0 Then
            num = num + 1
            aa$ = "Error" + LTrim$(Str$(num))
            abuf$(num) = aa$ + "  材料の密度が入力されていません！"
            If num = 10 Then
                Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                Exit Sub
            End If
        End If
        If IncVisc = 1 Then
            If Ctherm = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf$(num) = aa$ + "  材料の比熱が入力されていません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            If Rlamda = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf$(num) = aa$ + "  材料の熱伝導率が入力されていません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
            If Rmyu = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf$(num) = aa$ + "  材料の粘度（基準）が入力されていません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
        End If

        If Kvisc > 0 Then   '粘度式ﾃﾞｰﾀのﾁｪｯｸ
            Select Case Kvisc
                Case 1
                    jmax = 2
                Case 2
                    jmax = 5
                Case 3
                    jmax = 5
            End Select
            For j = 0 To jmax
                If Avisc(j) = 0 Then
                    num = num + 1
                    aa$ = "Error" + LTrim$(Str$(num))
                    aa$ = aa$ + "  粘度式番号" + Str$(Kvisc) + "で"
                    aa$ = aa$ + " 係数A" + LTrim$(Str$(j))
                    abuf$(num) = aa$ + "が入力されていません！"
                    If num = 10 Then
                        Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                        Exit Sub
                    End If
                End If
            Next j
        End If

        If KBodyForce > 0 Then  ' "物体力の加速度のﾁｪｯｸ
            If Accel(1) = 0 And Accel(2) = 0 And Accel(3) = 0 Then
                num = num + 1
                aa$ = "Error" + LTrim$(Str$(num))
                abuf$(num) = aa$ + "  物体力の加速度が入力されていません！"
                If num = 10 Then
                    Call ErrorDisp(1, num, abuf$) 'ｴﾗｰの表示
                    Exit Sub
                End If
            End If
        End If

        If num = 0 Then
            If kdisp = 1 Then
                aa$ = "データのエラーチェックが終了しました！"
                MsgBox(aa$, 0, "データＯＫ")
            End If
        Else
            Call ErrorDisp(0, num, abuf$) 'ｴﾗｰの表示
        End If

    End Sub

    Sub CratioInit()
        '追加係数初期設定
        Dim b0, b1 As Single

        Randomize()
        b0 = 0.01
        b1 = 0.1 * Rnd()
        If b1 < b0 Then
            b1 = b0
        End If
        Cratio = b1

    End Sub

    Private Sub CrosPoint(ByRef alphab As Single, ByRef beta As Single, _
    ByVal eps As Single, ByVal xbuf() As Single, ByVal ybuf() As Single, _
    ByVal xbufc() As Single, ByVal ybufc() As Single)
        '2線分の交点計算
        'alphab,beta･･･各線分の交点位置比
        'xbuf(),ybuf()･･･基準側の2節点座標
        'xbufc(),ybufc()･･･比較側の2節点座標
        Dim a1, a2, b1, b2, b3, b4, bb, cc, dd, dist, distb As Single
        Dim eps2, f1, f2, xb, yb As Single

        alphab = -1
        eps2 = eps * eps
        a1 = ybuf(2) - ybuf(1)
        b1 = -(xbuf(2) - xbuf(1))
        dist = Math.Sqrt(a1 * a1 + b1 * b1)
        a2 = ybufc(2) - ybufc(1)
        b2 = -(xbufc(2) - xbufc(1))
        distb = Math.Sqrt(a2 * a2 + b2 * b2)
        If dist < eps Or distb < eps Then
            Exit Sub
        End If
        If Math.Abs(a1) < eps And Math.Abs(b2) < eps Then
            xb = xbufc(1)
            yb = ybuf(1)
            b1 = xb - xbuf(1)
            b2 = yb - ybufc(1)
            cc = b1 * (xb - xbuf(2))
            dd = b2 * (yb - ybufc(2))
            If cc < eps2 And dd < eps2 Then
                alphab = Math.Abs(b1) / dist
                beta = Math.Abs(b2) / distb
            End If
        ElseIf Math.Abs(b1) < eps And Math.Abs(a2) < eps Then
            xb = xbuf(1)
            yb = ybufc(1)
            b1 = yb - ybuf(1)
            b2 = xb - xbufc(1)
            cc = b1 * (yb - ybuf(2))
            dd = b2 * (xb - xbufc(2))
            If cc < eps2 And dd < eps2 Then
                alphab = Math.Abs(b1) / dist
                beta = Math.Abs(b2) / distb
            End If
        Else
            f1 = a1 * xbuf(1) + b1 * ybuf(1)
            f2 = a2 * xbufc(1) + b2 * ybufc(1)
            bb = a1 * b2 - b1 * a2
            If Math.Abs(bb) > eps2 Then
                xb = (f1 * b2 - f2 * b1) / bb
                yb = (f2 * a1 - f1 * a2) / bb
                b1 = xb - xbuf(1)
                b2 = yb - ybuf(1)
                cc = b1 * (xb - xbuf(2))
                dd = b2 * (yb - ybuf(2))
                If cc < eps2 And dd < eps2 Then
                    b3 = xb - xbufc(1)
                    b4 = yb - ybufc(1)
                    cc = b3 * (xb - xbufc(2))
                    dd = b4 * (yb - ybufc(2))
                    If cc < eps2 And dd < eps2 Then
                        alphab = Math.Sqrt(b1 * b1 + b2 * b2) / dist
                        beta = Math.Sqrt(b3 * b3 + b4 * b4) / distb
                    End If
                End If
            End If
        End If

    End Sub

    Sub DataStoreCheck(ByRef ix As Integer)
        'ﾃﾞｰﾀ保存の確認
        'ix･･･返答（6:yes、それ以外は中止）
        ix = 0
        If KdataChange > 0 Then
            Dim aa$
            aa$ = Fm$ + "ファイルが変更されていますが、保存なしでよいですか？"
            ix = MsgBox(aa$, 4, "データ変更の警告")
            If ix = 6 Then
                KdataChange = 0
            End If
        End If

    End Sub

    Sub DfileInputU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim aa$
        Dim i, j, jb, jmax, nmax As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) <> "プログラム名" Then
            aa$ = "本ソフトのUTF-8データファイルとは違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        If dat(1) <> "Flow3DLiq" Then
            aa$ = "データファイルが違います！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindElem = dat(1)
        Title$ = sr.ReadLine
        If KindElem = 0 Then
            nmax = 20
        Else
            aa$ = "六面体要素以外は使用できません！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        IncVisc = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Npoint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nelem = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        NboundP = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        NboundS = dat(1)
        If IncVisc = 1 Then
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NBtempP = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NBtempS = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Nflux = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Kvisc = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            KBodyForce = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            InitDataV = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            InitDataT = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Ngaus = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Ndim = dat(1)
        Else
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Nflux = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            KBodyForce = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Ngaus = dat(1)
        End If

        ReDim NNelem(Nelem, nmax)
        ReDim Xdata(Npoint), Ydata(Npoint), Zdata(Npoint)
        ReDim NrestP(NboundP), JfixP(NboundP, 4), VfixP(NboundP, 4)
        ReDim KsurfRest(NboundS, 2), JfixS(NboundS, 4), VfixS(NboundS, 4)
        ReDim NTrestP(NBtempP), TempFixP(NBtempP)
        ReDim KsurfTrest(NBtempS, 2), TempFixS(NBtempS)
        ReDim KsurfFlux(Nflux, 2), Pnorm(Nflux)
        ReDim Vinit(Npoint, Ndim), TempInit(Npoint)

        aa$ = sr.ReadLine
        For i = 1 To Nelem
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            For j = 1 To 10
                NNelem(i, j) = dat(j)
            Next j
            If nmax > 10 Then
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                For j = 11 To nmax
                    NNelem(i, j) = dat(j - 11)
                Next j
            End If
        Next i

        aa$ = sr.ReadLine
        For i = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Xdata(i) = dat(1)
            Ydata(i) = dat(2)
            Zdata(i) = dat(3)
        Next i

        If NboundP > 0 Then
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            For i = 1 To NboundP
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                NrestP(i) = dat(1)
                If IncVisc = 0 Then
                    VfixP(i, 1) = dat(2)
                Else
                    For j = 1 To Ndim
                        jb = 2 * j
                        JfixP(i, j) = dat(jb)
                        VfixP(i, j) = dat(jb + 1)
                    Next j
                End If
            Next i
        End If

        If NboundS > 0 Then
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            For i = 1 To NboundS
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KsurfRest(i, 1) = dat(1)
                KsurfRest(i, 2) = dat(2)
                If IncVisc = 0 Then
                    VfixS(i, 1) = dat(3)
                Else
                    For j = 1 To Ndim
                        jb = 2 * j
                        JfixS(i, j) = dat(jb + 1)
                        VfixS(i, j) = dat(jb + 2)
                    Next j
                End If
            Next i
        End If

        If IncVisc = 1 Then
            If NBtempP > 0 Then
                aa$ = sr.ReadLine
                For i = 1 To NBtempP
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    NTrestP(i) = dat(1)
                    TempFixP(i) = dat(2)
                Next i
            End If

            If NBtempS > 0 Then
                aa$ = sr.ReadLine
                For i = 1 To NBtempS
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    KsurfTrest(i, 1) = dat(1)
                    KsurfTrest(i, 2) = dat(2)
                    TempFixS(i) = dat(3)
                Next i
            End If
        End If

        If Nflux > 0 Then
            aa$ = sr.ReadLine
            For i = 1 To Nflux
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KsurfFlux(i, 1) = dat(1)
                KsurfFlux(i, 2) = dat(2)
                Pnorm(i) = dat(3)
            Next i
        End If

        If IncVisc = 0 Then
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            KpressP = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            PressStd = dat(1)
        End If

        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Rou = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Ctherm = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Rlamda = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Rmyu = dat(1)

        If IncVisc = 1 Then
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            If Kvisc > 0 Then
                aa$ = sr.ReadLine
                Select Case Kvisc
                    Case 1
                        jmax = 2
                    Case 2
                        jmax = 5
                    Case 3
                        jmax = 5
                    Case 4
                        jmax = 2
                End Select
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                For j = 0 To jmax
                    Avisc(j) = dat(j)
                Next j
            End If
        End If

        If KBodyForce > 0 Then
            aa$ = sr.ReadLine
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Accel(1) = dat(0)
            Accel(2) = dat(1)
            Accel(3) = dat(2)
        Else
            Accel(1) = 0 : Accel(2) = 0 : Accel(3) = 0
        End If

        If IncVisc = 1 Then
            If InitDataV > 0 Then
                aa$ = sr.ReadLine
                For i = 1 To Npoint
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    For j = 1 To Ndim
                        Vinit(i, j) = dat(j)
                    Next j
                Next i
            End If

            If InitDataT > 0 Then
                aa$ = sr.ReadLine
                For i = 1 To Npoint
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    TempInit(i) = dat(1)
                Next i
            End If
        End If

        sr.Close()

    End Sub

    Sub DfileOutU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim a1$, aa$
        Dim i, j, jmax As Integer

        a1$ = ""
        Dim sw As New IO.StreamWriter(fxx$)
        aa$ = "プログラム名" + "," + "Flow3DLiq"
        sw.WriteLine(aa$)
        aa$ = "要素種類（0:六面体、1:四面体）" + "," + Str(KindElem)
        sw.WriteLine(aa$)
        sw.WriteLine(Title$)
        aa$ = "粘性の有無（0:なし、1:有り）" + "," + Str(IncVisc)
        sw.WriteLine(aa$)
        aa$ = "節点数" + "," + Str(Npoint)
        sw.WriteLine(aa$)
        aa$ = "要素数" + "," + Str(Nelem)
        sw.WriteLine(aa$)
        If IncVisc = 0 Then
            aa$ = "速度ポテンシャル固定の節点数" + "," + Str(NboundP)
            sw.WriteLine(aa$)
            aa$ = "速度ポテンシャルの面の数" + "," + Str(NboundS)
            sw.WriteLine(aa$)
            aa$ = "面垂直流の面の数" + "," + Str(Nflux)
            sw.WriteLine(aa$)
        Else
            aa$ = "速度・圧力固定の節点数" + "," + Str(NboundP)
            sw.WriteLine(aa$)
            aa$ = "速度・圧力固定の面の数" + "," + Str(NboundS)
            sw.WriteLine(aa$)
            aa$ = "温度固定節点数" + "," + Str(NBtempP)
            sw.WriteLine(aa$)
            aa$ = "温度固定の面の数" + "," + Str(NBtempS)
            sw.WriteLine(aa$)
            aa$ = "熱流束境界の面の数" + "," + Str(Nflux)
            sw.WriteLine(aa$)
            aa$ = "粘度式の種類" + "," + Str(Kvisc)
            sw.WriteLine(aa$)
        End If
        aa$ = "物体力の有無（0:なし、1:有り）" + "," + Str(KBodyForce)
        sw.WriteLine(aa$)
        If IncVisc = 1 Then
            aa$ = "速度・圧力の初期値データの有無" + "," + Str(InitDataV)
            sw.WriteLine(aa$)
            aa$ = "温度の初期値データの有無" + "," + Str(InitDataT)
            sw.WriteLine(aa$)
        End If
        Ngaus = 3 : Ndim = 4
        aa$ = "ガウス積分点数" + "," + Str(Ngaus)
        sw.WriteLine(aa$)
        If IncVisc = 1 Then
            aa$ = "１節点当りの変数の数" + "," + Str(Ndim)
            sw.WriteLine(aa$)
        End If

        If KindElem = 0 Then
            sw.WriteLine("要素データ (要素番号、20節点番号)")
        End If
        For i = 1 To Nelem
            aa$ = Str$(i)
            For j = 1 To 10
                aa$ = aa$ + "," + Str$(NNelem(i, j))
            Next j
            sw.WriteLine(aa$)
            If KindElem = 0 Then
                aa$ = "    " + Str$(NNelem(i, 11))
                For j = 12 To 20
                    aa$ = aa$ + "," + Str$(NNelem(i, j))
                Next j
                sw.WriteLine(aa$)
            End If
        Next i

        aa$ = "節点データ（節点番号、Ｘ座標、Ｙ座標、Ｚ座標）"
        sw.WriteLine(aa$)
        For i = 1 To Npoint
            aa$ = Str$(i)
            aa$ = aa$ + ",  " + Str$(Xdata(i))
            aa$ = aa$ + ",  " + Str$(Ydata(i))
            aa$ = aa$ + ",  " + Str$(Zdata(i))
            sw.WriteLine(aa$)
        Next i

        If NboundP > 0 Then
            If IncVisc = 0 Then
                sw.WriteLine("速度ポテンシャル固定の節点データ")
                aa$ = "（節点固定番号、節点番号、φ値）"
                sw.WriteLine(aa$)
                For i = 1 To NboundP
                    aa$ = Str$(i) + ",  " + Str$(NrestP(i))
                    aa$ = aa$ + ",   " + Str$(VfixP(i, 1))
                    sw.WriteLine(aa$)
                Next i
            Else
                sw.WriteLine("速度・圧力固定の節点データ")
                aa$ = "（節点固定番号、節点番号、U拘束、U値、V拘束、V値、"
                aa$ = aa$ + "W拘束、W値、P拘束、P値）"
                sw.WriteLine(aa$)
                For i = 1 To NboundP
                    aa$ = Str$(i) + ",  " + Str$(NrestP(i))
                    For j = 1 To Ndim
                        aa$ = aa$ + ",   " + Str$(JfixP(i, j))
                        aa$ = aa$ + ",   " + Str$(VfixP(i, j))
                    Next j
                    sw.WriteLine(aa$)
                Next i
            End If
        End If

        If NboundS > 0 Then
            If IncVisc = 0 Then
                sw.WriteLine("速度ポテンシャルの面のデータ")
                aa$ = "（面固定番号、要素番号、面番号、φ値）"
                sw.WriteLine(aa$)
                For i = 1 To NboundS
                    aa$ = Str$(i)
                    For j = 1 To 2
                        aa$ = aa$ + ",  " + Str$(KsurfRest(i, j))
                    Next j
                    aa$ = aa$ + ",   " + Str$(VfixS(i, 1))
                    sw.WriteLine(aa$)
                Next i

            Else
                sw.WriteLine("速度・圧力固定の面のデータ")
                aa$ = "（面固定番号、要素番号、面番号、U拘束、U値、"
                aa$ = aa$ + "V拘束、V値、W拘束、W値、P拘束、P値）"
                sw.WriteLine(aa$)
                For i = 1 To NboundS
                    aa$ = Str$(i)
                    For j = 1 To 2
                        aa$ = aa$ + ",  " + Str$(KsurfRest(i, j))
                    Next j
                    For j = 1 To Ndim
                        aa$ = aa$ + ",   " + Str$(JfixS(i, j))
                        aa$ = aa$ + ",   " + Str$(VfixS(i, j))
                    Next j
                    sw.WriteLine(aa$)
                Next i
            End If
        End If

        If IncVisc = 1 Then
            If NBtempP > 0 Then
                sw.WriteLine("温度固定の節点データ（節点固定番号、節点番号、温度）")
                For i = 1 To NBtempP
                    aa$ = Str$(i) + ",  " + Str$(NTrestP(i))
                    aa$ = aa$ + ",  " + Str$(TempFixP(i))
                    sw.WriteLine(aa$)
                Next i
            End If

            If NBtempS > 0 Then
                aa$ = "温度固定の面のデータ（面固定番号、要素番号、面番号、"
                aa$ = aa$ + "温度）"
                sw.WriteLine(aa$)
                For i = 1 To NBtempS
                    aa$ = Str$(i)
                    For j = 1 To 2
                        aa$ = aa$ + ",  " + Str$(KsurfTrest(i, j))
                    Next j
                    aa$ = aa$ + ",  " + Str$(TempFixS(i))
                    sw.WriteLine(aa$)
                Next i
            End If
        End If

        If Nflux > 0 Then
            If IncVisc = 0 Then
                aa$ = "面垂直流データ（設定番号、要素番号、面番号、垂直流速）"
                sw.WriteLine(aa$)
            Else
                aa$ = "熱流束データ（熱流束番号、要素番号、面番号、"
                aa$ = aa$ + "熱流束（単位面積当り））"
                sw.WriteLine(aa$)
            End If
            For i = 1 To Nflux
                aa$ = Str$(i)
                For j = 1 To 2
                    aa$ = aa$ + ",  " + Str$(KsurfFlux(i, j))
                Next j
                aa$ = aa$ + ",  " + Str$(Pnorm(i))
                sw.WriteLine(aa$)
            Next i
        End If

        If IncVisc = 0 Then
            aa$ = "基準圧力の節点番号" + "," + Str(KpressP)
            sw.WriteLine(aa$)
            aa$ = "基準圧力の値" + "," + Str(PressStd)
            sw.WriteLine(aa$)
        End If

        sw.WriteLine("材料データ")
        aa$ = "  密度＝" + "," + Str(Rou)
        sw.WriteLine(aa$)
        aa$ = "  比熱＝" + "," + Str(Ctherm)
        sw.WriteLine(aa$)
        aa$ = "  熱伝導率＝" + "," + Str(Rlamda)
        sw.WriteLine(aa$)
        aa$ = "  粘度（基準）＝" + "," + Str(Rmyu)
        sw.WriteLine(aa$)

        If IncVisc = 1 Then
            sw.WriteLine("粘度式")
            If Kvisc = 0 Then
                sw.WriteLine("  なし")
            Else
                Select Case Kvisc
                    Case 1
                        a1$ = "(1)η=A0*EXP(A1*Temp)*(GD^A2)"
                        aa$ = "  A0、A1、A2="
                        jmax = 2
                    Case 2
                        a1$ = "(2)LGT(η)=A0+A1*LGT(GD)+A2*Temp"
                        a1$ = a1$ + "+A3*(LGT(GD)^2)+A4*LGT(GD)*Temp+A5*(Temp^2)"
                        aa$ = "  A0、A1、A2、A3、A4、A5="
                        jmax = 5
                    Case 3
                        a1$ = "(3)LOG(η)=A0+A1*LOG(GD)+A2*Temp"
                        a1$ = a1$ + "+A3*(LOG(GD)^2)+A4*LOG(GD)*Temp+A5*(Temp^2)"
                        aa$ = "  A0、A1、A2、A3、A4、A5="
                        jmax = 5
                    Case 4
                        a1$ = "(4)η=A0+A1*Temp+A2+Temp^2"
                        aa$ = "  A0、A1、A2="
                        jmax = 2
                End Select
                sw.WriteLine(a1$)
                sw.WriteLine(aa$)
                aa$ = Str$(Avisc(0))
                For j = 1 To jmax
                    aa$ = aa$ + ",  " + Str$(Avisc(j))
                Next j
                sw.WriteLine(aa$)
            End If
        End If

        If KBodyForce > 0 Then
            sw.WriteLine("物体力の加速度（X方向、Y方向、Z方向）")
            aa$ = Str(Accel(1)) + ",  " + Str(Accel(2)) + ",  " + Str(Accel(3))
            sw.WriteLine(aa$)
        End If

        If IncVisc = 1 Then
            If InitDataV > 0 Then
                aa$ = "節点番号、初期速度U、初期速度V、初期速度W、初期圧力P"
                sw.WriteLine(aa$)
                For i = 1 To Npoint
                    aa$ = Str(i)
                    For j = 1 To Ndim
                        aa$ = aa$ + ",  " + Str(Vinit(i, j))
                    Next j
                    sw.WriteLine(aa$)
                Next i
            End If

            If InitDataT > 0 Then
                aa$ = "節点番号、初期温度T"
                sw.WriteLine(aa$)
                For i = 1 To Npoint
                    aa$ = Str(i)
                    aa$ = aa$ + ",  " + Str(TempInit(i))
                    sw.WriteLine(aa$)
                Next i
            End If
        End If

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Private Sub DistComp(ByRef ic As Integer, ByVal kbuf() As Integer)
        '距離比較
        'ic･･･基準面の距離区分（0:近い、1:遠い）
        'kbuf()･･･比較面の8節点番号
        Dim i, i1, i2, ib, icc, j, j1, j2, jb, nmax As Integer
        Dim alphab, b1, b2, b3, beta, c1, dist, distc, eps, xb, yb, zb As Single
        Dim xbuf(2), xbufc(2), ybuf(2), ybufc(2) As Single

        If KindElem = 0 Then
            nmax = 8
            c1 = 0.25
        End If
        ic = -1  '仮設定
        For i = 1 To nmax - 1 Step 2
            i1 = KsurfPoint(i)
            ib = i + 2
            If ib > nmax Then
                ib = 1
            End If
            i2 = KsurfPoint(ib)
            xbuf(1) = Xscreen(i1)
            ybuf(1) = Yscreen(i1)
            xbuf(2) = Xscreen(i2)
            ybuf(2) = Yscreen(i2)
            b1 = Xmodel(i1) - Xmodel(i2)
            b2 = Ymodel(i1) - Ymodel(i2)
            b3 = Zmodel(i1) - Zmodel(i2)
            eps = 0.0001 * Math.Sqrt(b1 * b1 + b2 * b2 + b3 * b3)
            For j = 1 To nmax - 1 Step 2
                j1 = kbuf(j)
                jb = j + 2
                If jb > nmax Then
                    jb = 1
                End If
                j2 = kbuf(jb)
                icc = 0
                If i1 = j1 Or i1 = j2 Then
                    icc = 1
                ElseIf i2 = j1 Or i2 = j2 Then
                    icc = 1
                End If
                If icc = 0 Then
                    xbufc(1) = Xscreen(j1)
                    ybufc(1) = Yscreen(j1)
                    xbufc(2) = Xscreen(j2)
                    ybufc(2) = Yscreen(j2)
                    Call CrosPoint(alphab, beta, eps, xbuf, ybuf, xbufc, ybufc)
                    '2線分の交点計算

                    If alphab >= 0 Then
                        b1 = 1 - alphab
                        xb = b1 * Xmodel(i1) + alphab * Xmodel(i2)
                        yb = b1 * Ymodel(i1) + alphab * Ymodel(i2) + Rlenmodel
                        zb = b1 * Zmodel(i1) + alphab * Zmodel(i2) - Eyeh
                        dist = xb * xb + yb * yb + zb * zb
                        b1 = 1 - beta
                        xb = b1 * Xmodel(j1) + beta * Xmodel(j2)
                        yb = b1 * Ymodel(j1) + beta * Ymodel(j2) + Rlenmodel
                        zb = b1 * Zmodel(j1) + beta * Zmodel(j2) - Eyeh
                        distc = xb * xb + yb * yb + zb * zb
                        If dist < distc Then
                            ic = 0
                        Else
                            ic = 1
                        End If
                        icc = -1
                        Exit For
                    End If
                End If
            Next j
            If icc < 0 Then
                Exit For
            End If
        Next i

        If ic < 0 Then
            xb = 0 : yb = 0 : zb = 0
            For i = 1 To nmax - 1 Step 2
                ib = KsurfPoint(i)
                xb = xb + Xmodel(ib)
                yb = yb + Ymodel(ib)
                zb = zb + Zmodel(ib)
            Next
            xb = c1 * xb
            yb = c1 * yb + Rlenmodel
            zb = c1 * zb - Eyeh
            dist = xb * xb + yb * yb + zb * zb
            xb = 0 : yb = 0 : zb = 0
            For i = 1 To nmax - 1 Step 2
                ib = kbuf(i)
                xb = xb + Xmodel(ib)
                yb = yb + Ymodel(ib)
                zb = zb + Zmodel(ib)
            Next
            xb = c1 * xb
            yb = c1 * yb + Rlenmodel
            zb = c1 * zb - Eyeh
            distc = xb * xb + yb * yb + zb * zb

            If dist < distc Then
                ic = 0
            Else
                ic = 1
            End If
        End If

    End Sub

    Sub DrawFocusRect(ByRef gr As Graphics)
        'ﾌｫｰｶｽ矩形の描画
        Dim rect As Rectangle
        Dim p1, p2 As Point

        If Xmouse(1) <= Xmouse(2) Then
            p1.X = Xmouse(1)
            p2.X = Xmouse(2)
        Else
            p1.X = Xmouse(2)
            p2.X = Xmouse(1)
        End If
        If Ymouse(1) <= Ymouse(2) Then
            p1.Y = Ymouse(1)
            p2.Y = Ymouse(2)
        Else
            p1.Y = Ymouse(2)
            p2.Y = Ymouse(1)
        End If
        rect.X = p1.X
        rect.Y = p1.Y
        rect.Width = p2.X - p1.X
        rect.Height = p2.Y - p1.Y

        gr.DrawRectangle(Pens.DarkGray, rect)   '矩形の描画

    End Sub

    Sub DrawNodeVar(ByRef gr As Graphics, ByVal aa As String, _
    ByVal xb As Single, ByVal yb As Single)
        '節点変数の描画
        'aa･･･変数のテキスト
        'xb,yb･･･描画位置
        Dim xbw, ybw As Single
        Dim fnt2 As Font = New Font("ＭＳ ゴシック", 10, FontStyle.Bold)

        xbw = xb - 1
        ybw = yb
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        xbw = xb + 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        xbw = xb
        ybw = yb - 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        ybw = yb + 1
        gr.DrawString(aa$, fnt2, Brushes.White, xbw, ybw)
        gr.DrawString(aa$, fnt2, Brushes.Purple, xb, yb)

    End Sub

    Sub DrawScale(ByVal dpixb As Single, ByVal dpiyb As Single, _
    ByVal xlen As Single, ByVal ylen As Single, ByVal xmin As Single, _
    ByVal xmax As Single, ByVal ymin As Single, ByVal ymax As Single)
        '作画倍率の計算
        'dpixb･･･X軸方向のdpi値
        'dpiyb･･･Y軸方向のdpi値
        'xlen･･･作画領域のX方向長さ
        'ylen･･･作画領域のY方向長さ
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim b1, b2, cx, cy As Single

        Cfix = dpiyb / dpixb

        b1 = xmax - xmin : b2 = ymax - ymin
        If Cfix < 1 Then
            If b1 <> 0 Then
                cx = xlen / b1
            End If
            If b2 <> 0 Then
                cy = ylen / b2 / Cfix
            End If
        Else
            If b1 <> 0 Then
                cx = Cfix * xlen / b1
            End If
            If b2 <> 0 Then
                cy = ylen / b2
            End If
        End If
        If b1 = 0 Then
            XYscale = cy
        ElseIf b2 = 0 Then
            XYscale = cx
        ElseIf cy < cx Then
            XYscale = cy
        Else
            XYscale = cx
        End If

    End Sub

    Sub Elem3DPrep()
        '構造図作画準備
        Dim ie As Integer
        ReDim KdrawElem(Nelem)
        ReDim Xmodel(Npoint), Ymodel(Npoint), Zmodel(Npoint)
        ReDim Xscreen(Npoint), Yscreen(Npoint)
        ReDim Xview(Npoint), Yview(Npoint), Zview(Npoint)

        KE1 = 1
        KE2 = Nelem
        KnumDisp = 0
        KnumSize = 8
        For ie = KE1 To KE2
            KdrawElem(ie) = 1
        Next ie

        KrearTreat = 1
        KnumDisp = 0
        KdispENum = 0
        KdispPNum = 0
        KdispSNum = 0

        Rlenmax = 0
        DistFactor = 0      '作画ﾊﾟﾗﾒｰﾀｰのｸﾘｱ

    End Sub

    Private Sub ElemCalc(ByVal ie As Integer, ByRef vbuf As Single)
        '六面体要素の計算
        'ie･･･要素番号
        'vbuf･･･体積
        Dim i, ig, ii, jg, kg, numg As Integer
        Dim dv, s, t, u As Single
        Dim dj As Double

        For ii = 1 To 20
            i = NNelem(ie, ii)
            XE(ii) = Xdata(i)
            YE(ii) = Ydata(i)
            ZE(ii) = Zdata(i)
        Next ii

        vbuf = 0
        numg = 0
        For ig = 1 To Ngaus
            s = PosGaus(ig)
            For jg = 1 To Ngaus
                t = PosGaus(jg)
                For kg = 1 To Ngaus
                    numg = numg + 1
                    u = PosGaus(kg)
                    Call ShapeFunc(1, s, t, u)         '形状関数の計算
                    Call JacobElem(ie, 1, numg, dj)   '要素のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算
                    dv = dj * WeightG(ig) * WeightG(jg) * WeightG(kg)
                    vbuf = vbuf + dv
                Next kg
            Next jg
        Next ig

    End Sub

    Sub ElemRelation(ByRef icc As Integer, ByVal ie As Integer, ByVal ib As Integer)
        '要素関係の判別
        'icc･･･関係の有無（0:なし、1:有り）
        'ie･･･元の要素番号
        'ib･･･対象の要素番号
        Dim i, j, k As Integer
        If KindElem = 0 Then
            For i = 1 To 7 Step 2
                k = NNelem(ie, i)
                For j = 1 To 7 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
                For j = 13 To 19 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
            Next i
            For i = 13 To 19 Step 2
                k = NNelem(ie, i)
                For j = 1 To 7 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
                For j = 13 To 19 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
            Next i
        End If

    End Sub

    Sub ElemSearch(ByRef k1 As Integer, ByRef k2 As Integer)
        '拡大作画する要素の探索
        '全節点が指定領域にある要素を採用、作画要素なしの場合は元のﾃﾞｰﾀを保存
        'k1,k2･･･要素の開始、終了番号
        Dim ic, ie, j, k, kmin, kmax, nmax As Integer
        Dim b1, b2 As Single
        Dim kbuf(Nelem) As Integer
        If KindElem = 0 Then
            nmax = 20
        End If
        ic = 0
        For ie = 1 To Nelem
            kbuf(ie) = KdrawElem(ie)
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    b1 = Xscreen(k) - Xmouse(1)
                    b2 = Xscreen(k) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        b1 = Yscreen(k) - Ymouse(2)
                        b2 = Yscreen(k) - Ymouse(1)
                        If b1 >= 0 And b2 <= 0 Then
                            If j = nmax Then
                                KdrawElem(ie) = 1
                                If ic = 0 Then
                                    ic = 1
                                    kmin = ie
                                    kmax = ie
                                ElseIf ie < kmin Then
                                    kmin = ie
                                ElseIf ie > kmax Then
                                    kmax = ie
                                End If
                            End If
                        Else
                            KdrawElem(ie) = 0
                            Exit For
                        End If
                    Else
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie
        k1 = kmin
        k2 = kmax
        If k1 = 0 Then
            For ie = 1 To Nelem
                KdrawElem(ie) = kbuf(ie)
            Next ie
        End If

    End Sub

    Sub ElemSearch3(ByRef nd As Integer)
        '矩形領域の外周面の探索
        'k1,k2･･･要素の開始、終了番号
        'nb･･･適合面の数
        Dim ie, j, jmax, k, kb, kk, kmax, nb As Integer
        Dim b1, b2 As Single
        If KindElem = 0 Then
            jmax = 6
            kmax = 8
        End If
        For ie = 1 To Nelem
            For j = 1 To jmax
                KAppSide(ie, j) = 0
            Next j
        Next ie
        nd = 0
        For nb = 1 To NSdraw
            ie = KEorder(nb)
            kb = KSorder(nb)
            If KEInside(ie, kb) = 0 Then
                If KindElem = 0 Then
                    Call SurfaceSet(ie, kb, KsurfPoint)     '面の設定
                End If
                For k = 1 To kmax
                    kk = KsurfPoint(k)
                    b1 = Xscreen(kk) - Xmouse(1)
                    b2 = Xscreen(kk) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        b1 = Yscreen(kk) - Ymouse(2)
                        b2 = Yscreen(kk) - Ymouse(1)
                        If b1 >= 0 And b2 <= 0 Then
                            If k = kmax Then
                                KAppSide(ie, kb) = 1
                                nd = nd + 1
                            End If
                        Else
                            Exit For
                        End If
                    Else
                        Exit For
                    End If
                Next k
            End If
        Next nb

    End Sub

    Sub EyeCheck(ByVal k1 As Integer, ByVal k2 As Integer)
        '視線ﾁｪｯｸ（面の中心位置で判定）
        'k1,k2･･･開始・終了要素番号
        Dim ie, j, k, kb, kmax, nb, nmax As Integer
        Dim b1, b2, b3, b4, xb, yb, zb As Single
        Dim eyevec(3), planevec(3) As Single
        Dim xbufb(3), ybufb(3), zbuf(3) As Single

        If KindElem = 0 Then
            kmax = 6
            nmax = 8
            nb = 4
        End If
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If KSelem(ie, k) = 0 Then
                        If KindElem = 0 Then
                            Call SurfaceSet(ie, k, KsurfPoint)  '面の設定
                        End If

                        xb = 0 : yb = 0 : zb = 0
                        For j = 2 To nmax Step 2
                            kb = KsurfPoint(j)
                            xb = xb + Xmodel(kb)
                            yb = yb + Ymodel(kb)
                            zb = zb + Zmodel(kb) - Eyeh
                            Select Case j
                                Case 2
                                    xbufb(2) = Xmodel(kb)
                                    ybufb(2) = Ymodel(kb)
                                    zbuf(2) = Zmodel(kb) - Eyeh
                                Case 4
                                    xbufb(3) = Xmodel(kb)
                                    ybufb(3) = Ymodel(kb)
                                    zbuf(3) = Zmodel(kb) - Eyeh
                            End Select
                        Next j
                        xbufb(1) = xb / nb
                        ybufb(1) = yb / nb
                        zbuf(1) = zb / nb

                        '逆視線ﾍﾞｸﾄﾙの計算
                        b1 = -xbufb(1)
                        b2 = -(Rlenmodel + ybufb(1))
                        b3 = -zbuf(1)
                        b4 = Math.Sqrt(b1 * b1 + b2 * b2 + b3 * b3)
                        eyevec(1) = b1 / b4
                        eyevec(2) = b2 / b4
                        eyevec(3) = b3 / b4

                        '面の法線ﾍﾞｸﾄﾙの計算
                        b1 = (ybufb(2) - ybufb(1)) * (zbuf(3) - zbuf(1))
                        b1 = b1 - (zbuf(2) - zbuf(1)) * (ybufb(3) - ybufb(1))
                        b2 = (zbuf(2) - zbuf(1)) * (xbufb(3) - xbufb(1))
                        b2 = b2 - (xbufb(2) - xbufb(1)) * (zbuf(3) - zbuf(1))
                        b3 = (xbufb(2) - xbufb(1)) * (ybufb(3) - ybufb(1))
                        b3 = b3 - (ybufb(2) - ybufb(1)) * (xbufb(3) - xbufb(1))
                        b4 = Math.Sqrt(b1 * b1 + b2 * b2 + b3 * b3)
                        planevec(1) = b1 / b4
                        planevec(2) = b2 / b4
                        planevec(3) = b3 / b4

                        '逆視線ﾍﾞｸﾄﾙと面法線ﾍﾞｸﾄﾙのｽｶﾗｰ積計算
                        b1 = 0
                        For j = 1 To 3
                            b1 = b1 + eyevec(j) * planevec(j)
                        Next j
                        If b1 <= 0 Then
                            KSelem(ie, k) = -1   '視線外
                        End If
                    End If
                Next k
            End If
        Next ie

    End Sub

    Private Sub ErrorDisp(ByVal kind As Integer, ByVal num As Integer,
        ByRef abuf() As String)
        'ｴﾗｰの表示
        'kind･･･打切り表示の有無（0:なし、1:有り）
        'num･･･ｴﾗｰの個数
        'abuf$()･･･ｴﾗｰの表示文
        Dim aa$, nc$
        Dim i As Integer
        nc$ = Chr(13) + Chr(10)
        aa$ = ""
        For i = 1 To num
            aa = aa$ + abuf(i) + nc$
        Next i
        If kind > 0 Then
            aa$ = aa$ + "データチェックを打切りました！！"
        End If
        MsgBox(aa$, 0, "データエラー")

    End Sub

    Sub FDatDupCheck(ByRef nd As Integer)
        '熱流束の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim ksbuf(Nflux) As Integer

        nd = 0
        For i = 1 To Nflux - 1
            If ksbuf(i) = 0 Then
                For ii = i + 1 To Nflux
                    If KsurfFlux(i, 1) = KsurfFlux(ii, 1) Then
                        If KsurfFlux(i, 2) = KsurfFlux(ii, 2) Then
                            nd = nd + 1
                            ksbuf(ii) = -1
                            Pnorm(i) = Pnorm(ii)
                        End If
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(Nflux, 2) As Integer
            Dim buf2(Nflux) As Single
            For i = 1 To Nflux
                For j = 1 To 2
                    nbuf1(i, j) = KsurfFlux(i, j)
                Next j
                buf2(i) = Pnorm(i)
            Next i
            nb = Nflux - nd
            ReDim KsurfFlux(nb, 2), Pnorm(nb)
            nb = 0
            For i = 1 To Nflux
                If ksbuf(i) = 0 Then
                    nb = nb + 1
                    KsurfFlux(nb, 1) = nbuf1(i, 1)
                    KsurfFlux(nb, 2) = nbuf1(i, 2)
                    Pnorm(nb) = buf2(i)
                End If
            Next i
            Nflux = nb
            Nfluxb = nb
        End If

    End Sub

    Sub FileName(ByVal fxx$, ByRef fxb$)
        'ﾌｧｲﾙ名fxx$の拡張子なしの部分fxb$を取り出す
        Dim i As Integer
        i = InStr(fxx$, ".")
        If i > 0 Then
            fxb$ = Left$(fxx$, i - 1)
        Else
            fxb$ = fxx$
        End If
    End Sub

    Function FreeMemoryGet() As Single
        '空きﾒﾓﾘｰ容量の取得
        'freemem･･･ﾒﾓﾘｰ空き容量（KB）
        Dim memfree As New System.Diagnostics.PerformanceCounter

        memfree.CategoryName = "Memory"
        memfree.CounterName = "Available KBytes"
        Return memfree.NextValue()

    End Function

    Function Ftrans(ByVal b1 As Single) As Single
        '伝達関数の計算
        'b1･･･元変数

        GoTo Skip1
        'ステップ関数
        If b1 >= 0 Then
            Return 1.0
        Else
            Return 0.0
        End If
Skip1:
        'GoTo Skip2
        'シグモイド関数
        Dim b2 As Single
        b2 = 1.0 / (1.0 + Math.Exp(-b1))
        Return b2
Skip2:

    End Function

    Sub GausSetStd()
        'ｶﾞｳｽ点の設定
        PosGaus(3) = 0.77459667 : PosGaus(1) = -PosGaus(3)
        PosGaus(2) = 0
        WeightG(1) = 0.55555556 : WeightG(3) = WeightG(1)
        WeightG(2) = 0.88888889

    End Sub

    Sub InnerCheck2(ByVal k1 As Integer, ByVal k2 As Integer)
        '内面ﾁｪｯｸ
        'k1,k2･･･開始・終了要素番号
        Dim ib, ic, icc, ie, ik, ik1, j, jj, jx, k, kk, kmax, ksb, ksmax, m, nmax As Integer
        Dim kbuf(8) As Integer

        If KindElem = 0 Then
            kmax = 6
            ksmax = 4
            nmax = 7
            ReDim KinnerLine(12 * Nelem) '内部辺のﾌﾗｯｸﾞ
        End If
        Dim ksbuf(Nelem, kmax) As Integer    'ﾁｪｯｸ済みﾌﾗｯｸﾞ
        For ie = k1 To k2 - 1
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If k = 1 Then
                        icc = 0
                    End If
                    If ksbuf(ie, k) = 0 Then
                        If KindElem = 0 Then
                            Call SurfaceSet(ie, k, KsurfPoint)  '面の設定
                        End If
                        ik = 0
                        For ib = ie + 1 To k2
                            If KdrawElem(ib) > 0 Then
                                If icc = 0 Then
                                    Call ElemRelation(icc, ie, ib)    '要素関係の判別
                                End If
                                If icc > 0 Then
                                    For kk = 1 To kmax
                                        If ksbuf(ib, kk) = 0 Then
                                            If KindElem = 0 Then
                                                Call SurfaceSet(ib, kk, kbuf)   '面の設定
                                            End If

                                            ik1 = KsurfPoint(1)
                                            ic = 0
                                            For j = 1 To nmax Step 2
                                                If ik1 = kbuf(j) Then
                                                    jx = j
                                                    For jj = 3 To nmax Step 2
                                                        jx = jx - 2
                                                        If jx < 1 Then
                                                            jx = nmax
                                                        End If
                                                        If KsurfPoint(jj) <> kbuf(jx) Then
                                                            ic = 1
                                                            Exit For
                                                        End If
                                                    Next jj
                                                    If ic = 0 Then
                                                        ik = 1
                                                        KSelem(ie, k) = 1
                                                        KSelem(ib, kk) = 1
                                                        ksbuf(ie, k) = 1
                                                        ksbuf(ib, kk) = 1
                                                        For m = 1 To ksmax
                                                            ksb = KSideLine(ie, k, m)
                                                            KinnerLine(ksb) = 1
                                                            ksb = KSideLine(ib, kk, m)
                                                            KinnerLine(ksb) = 1
                                                        Next m
                                                    End If
                                                    Exit For
                                                End If
                                            Next j
                                            If ik = 1 Then
                                                Exit For
                                            End If
                                        End If
                                    Next kk
                                    If ik = 1 Then
                                        Exit For
                                    End If
                                End If
                            Else
                                icc = 0
                            End If
                        Next ib
                    End If
                Next k
            End If
        Next ie

    End Sub

    Private Sub InnerCheck2S()
        '内面ﾁｪｯｸ
        Dim ib, ic, ie, ik, ik1, j, jj, jx, k, kk, kmax, nmax As Integer
        Dim kbuf(8) As Integer

        If KindElem = 0 Then
            kmax = 6
            nmax = 8
        End If
        Dim ksbuf(Nelem, kmax) As Integer    'ﾁｪｯｸ済みﾌﾗｯｸﾞ
        ReDim KSelem(Nelem, kmax)
        For ie = 1 To Nelem
            For k = 1 To kmax
                If ksbuf(ie, k) = 0 Then
                    If KindElem = 0 Then
                        Call SurfaceSet(ie, k, KsurfPoint)  '面の設定
                    End If
                    ik = 0
                    For ib = ie + 1 To Nelem
                        For kk = 1 To kmax
                            If ksbuf(ib, kk) = 0 Then
                                If KindElem = 0 Then
                                    Call SurfaceSet(ib, kk, kbuf)   '面の設定
                                End If
                                ik1 = KsurfPoint(1)
                                ic = 0
                                For j = 1 To nmax
                                    If ik1 = kbuf(j) Then
                                        jx = j
                                        For jj = 2 To nmax
                                            jx = jx - 1
                                            If jx < 1 Then
                                                jx = nmax
                                            End If
                                            If KsurfPoint(jj) <> kbuf(jx) Then
                                                ic = 1
                                                Exit For
                                            End If
                                        Next jj
                                        If ic = 0 Then
                                            ik = 1
                                            KSelem(ie, k) = 1
                                            KSelem(ib, kk) = 1
                                            ksbuf(ie, k) = 1
                                            ksbuf(ib, kk) = 1
                                            Exit For
                                        End If
                                    End If
                                Next j
                                If ik = 1 Then
                                    Exit For
                                End If
                            End If
                        Next kk
                        If ik = 1 Then
                            Exit For
                        End If
                    Next ib
                End If
            Next k
        Next ie

    End Sub

    Sub JacobElem(ByVal ie As Integer, ByVal kgaus As Integer, ByVal numg As Integer, _
        ByRef dj As Double)
        '要素のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算
        'ie･･･要素番号
        'kgaus･･･ｶﾞｳｽ点座標計算の有無（0:なし、1:有り）
        'numg･･･ｶﾞｳｽ点番号
        'dj･･･ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの行列式値
        Dim c1$, c2$
        Dim i, ii, j, nmax As Integer
        Dim b1 As Double

        If KindElem = 0 Then
            nmax = 20
        End If
        For i = 1 To 3
            For j = 1 To 3
                RinvJ(i, j) = 0
                RJmat(i, j) = 0
            Next j
        Next i
        If kgaus > 0 Then
            XG(numg) = 0 : YG(numg) = 0 : ZG(numg) = 0
            UG(numg) = 0 : VG(numg) = 0 : WG(numg) = 0
            TG(numg) = 0
            For ii = 1 To nmax
                XG(numg) = XG(numg) + XE(ii) * Shape(ii)
                YG(numg) = YG(numg) + YE(ii) * Shape(ii)
                ZG(numg) = ZG(numg) + ZE(ii) * Shape(ii)
                UG(numg) = UG(numg) + UE(ii) * Shape(ii)
                VG(numg) = VG(numg) + VE(ii) * Shape(ii)
                WG(numg) = WG(numg) + WE(ii) * Shape(ii)
                TG(numg) = TG(numg) + TE(ii) * Shape(ii)
            Next ii
        End If

        For ii = 1 To nmax
            For j = 1 To 3
                Select Case j
                    Case 1
                        b1 = XE(ii)
                    Case 2
                        b1 = YE(ii)
                    Case 3
                        b1 = ZE(ii)
                End Select
                RJmat(1, j) = RJmat(1, j) + Ds(ii) * b1
                RJmat(2, j) = RJmat(2, j) + Dt(ii) * b1
                RJmat(3, j) = RJmat(3, j) + Du(ii) * b1
            Next j
        Next ii
        RinvJ(1, 1) = RJmat(2, 2) * RJmat(3, 3) - RJmat(2, 3) * RJmat(3, 2)
        RinvJ(1, 2) = RJmat(1, 3) * RJmat(3, 2) - RJmat(1, 2) * RJmat(3, 3)
        RinvJ(1, 3) = RJmat(1, 2) * RJmat(2, 3) - RJmat(1, 3) * RJmat(2, 2)
        RinvJ(2, 1) = RJmat(2, 3) * RJmat(3, 1) - RJmat(2, 1) * RJmat(3, 3)
        RinvJ(2, 2) = RJmat(1, 1) * RJmat(3, 3) - RJmat(1, 3) * RJmat(3, 1)
        RinvJ(2, 3) = RJmat(1, 3) * RJmat(2, 1) - RJmat(1, 1) * RJmat(2, 3)
        RinvJ(3, 1) = RJmat(2, 1) * RJmat(3, 2) - RJmat(2, 2) * RJmat(3, 1)
        RinvJ(3, 2) = RJmat(1, 2) * RJmat(3, 1) - RJmat(1, 1) * RJmat(3, 2)
        RinvJ(3, 3) = RJmat(1, 1) * RJmat(2, 2) - RJmat(1, 2) * RJmat(2, 1)
        b1 = RJmat(1, 1) * RinvJ(1, 1) + RJmat(1, 2) * RinvJ(2, 1)
        dj = b1 + RJmat(1, 3) * RinvJ(3, 1)

        If dj <= 0 Then
            c1$ = "計算実行エラー"
            c2$ = "要素" + LTrim$(Str$(ie)) + "（ガウス点" + LTrim$(Str$(numg))
            c2$ = c2$ + "）でヤコビアン体積が負か０になりました。"
            MsgBox(c2$, 0, c1$)
            End
        Else
            '本来の逆ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽは下記のようにdjで割った値である。
            For i = 1 To 3
                For j = 1 To 3
                    RinvJT(i, j) = RinvJ(i, j) / dj
                Next j
            Next i
            For ii = 1 To nmax
                For i = 1 To 3
                    b1 = RinvJ(i, 1) * Ds(ii) + RinvJ(i, 2) * Dt(ii)
                    b1 = (b1 + RinvJ(i, 3) * Du(ii)) / dj
                    Select Case i
                        Case 1
                            Dx(ii) = b1
                        Case 2
                            Dy(ii) = b1
                        Case 3
                            Dz(ii) = b1
                    End Select
                Next i
            Next ii
        End If

    End Sub

    Sub JacobElem2(ByVal ie As Integer, ByVal numg As Integer, _
    ByRef dj As Double, ByVal knode() As Integer)
        '要素表面のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算
        'ie･･･要素番号
        'numg･･･ｶﾞｳｽ点番号
        'dj･･･ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの行列式値
        'knode()･･･節点の順位
        Dim i, ii, ip, j, kmax As Integer
        Dim b1 As Double

        If KindElem = 0 Then
            kmax = 8
        End If
        For i = 1 To 3
            For j = 1 To 3
                RinvJ(i, j) = 0
                RJmat(i, j) = 0
            Next j
        Next i
        '    XG(numg) = 0
        '    YG(numg) = 0
        '    ZG(numg) = 0
        '    For ip = 1 To 8
        '        ii = knode(ip)
        '        XG(numg) = XG(numg) + XE(ii) * Shape(ii)
        '        YG(numg) = YG(numg) + YE(ii) * Shape(ii)
        '        ZG(numg) = ZG(numg) + ZE(ii) * Shape(ii)
        '    Next ip
        For ip = 1 To kmax
            ii = knode(ip)
            For j = 1 To 3
                Select Case j
                    Case 1
                        b1 = XE(ii)
                    Case 2
                        b1 = YE(ii)
                    Case 3
                        b1 = ZE(ii)
                End Select
                RJmat(1, j) = RJmat(1, j) + Ds(ii) * b1
                RJmat(2, j) = RJmat(2, j) + Dt(ii) * b1
                RJmat(3, j) = RJmat(3, j) + Du(ii) * b1
            Next j
        Next ip

        '逆ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの配列に、面積分で使用するための値を保存
        RinvJ(1, 1) = RJmat(2, 2) * RJmat(3, 3) - RJmat(2, 3) * RJmat(3, 2)
        RinvJ(1, 2) = RJmat(1, 3) * RJmat(3, 2) - RJmat(1, 2) * RJmat(3, 3)
        RinvJ(1, 3) = RJmat(1, 2) * RJmat(2, 3) - RJmat(1, 3) * RJmat(2, 2)
        RinvJ(2, 1) = RJmat(2, 3) * RJmat(3, 1) - RJmat(2, 1) * RJmat(3, 3)
        RinvJ(2, 2) = RJmat(1, 1) * RJmat(3, 3) - RJmat(1, 3) * RJmat(3, 1)
        RinvJ(2, 3) = RJmat(1, 3) * RJmat(2, 1) - RJmat(1, 1) * RJmat(2, 3)
        RinvJ(3, 1) = RJmat(2, 1) * RJmat(3, 2) - RJmat(2, 2) * RJmat(3, 1)
        RinvJ(3, 2) = RJmat(1, 2) * RJmat(3, 1) - RJmat(1, 1) * RJmat(3, 2)
        RinvJ(3, 3) = RJmat(1, 1) * RJmat(2, 2) - RJmat(1, 2) * RJmat(2, 1)

    End Sub

    Sub JacobElem2S(ByVal ic As Integer, ByVal ie As Integer, _
    ByVal numg As Integer, ByRef dj As Double, ByVal knode() As Integer)
        '要素表面のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算
        'ic･･･計算区分（0:ｶﾞｳｽ点値のみ、1:全部）
        'ie･･･要素番号
        'numg･･･ｶﾞｳｽ点番号
        'dj･･･ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの行列式値
        'knode()･･･節点の順位
        Dim i, ii, ip, j, kmax As Integer
        Dim b1 As Double

        If KindElem = 0 Then
            kmax = 8
        End If
        For i = 1 To 3
            For j = 1 To 3
                RinvJ(i, j) = 0
                RJmat(i, j) = 0
            Next j
        Next i
        XG(numg) = 0
        YG(numg) = 0
        ZG(numg) = 0
        UG(numg) = 0
        VG(numg) = 0
        WG(numg) = 0
        For ip = 1 To kmax
            ii = knode(ip)
            XG(numg) = XG(numg) + XE(ii) * Shape(ii)
            YG(numg) = YG(numg) + YE(ii) * Shape(ii)
            ZG(numg) = ZG(numg) + ZE(ii) * Shape(ii)
            UG(numg) = UG(numg) + UE(ii) * Shape(ii)
            VG(numg) = VG(numg) + VE(ii) * Shape(ii)
            WG(numg) = WG(numg) + WE(ii) * Shape(ii)
        Next ip
        If ic > 0 Then
            For ip = 1 To kmax
                ii = knode(ip)
                For j = 1 To 3
                    Select Case j
                        Case 1
                            b1 = XE(ii)
                        Case 2
                            b1 = YE(ii)
                        Case 3
                            b1 = ZE(ii)
                    End Select
                    RJmat(1, j) = RJmat(1, j) + Ds(ii) * b1
                    RJmat(2, j) = RJmat(2, j) + Dt(ii) * b1
                    RJmat(3, j) = RJmat(3, j) + Du(ii) * b1
                Next j
            Next ip
            RinvJ(1, 1) = RJmat(2, 2) * RJmat(3, 3) - RJmat(2, 3) * RJmat(3, 2)
            RinvJ(1, 2) = RJmat(1, 3) * RJmat(3, 2) - RJmat(1, 2) * RJmat(3, 3)
            RinvJ(1, 3) = RJmat(1, 2) * RJmat(2, 3) - RJmat(1, 3) * RJmat(2, 2)
            RinvJ(2, 1) = RJmat(2, 3) * RJmat(3, 1) - RJmat(2, 1) * RJmat(3, 3)
            RinvJ(2, 2) = RJmat(1, 1) * RJmat(3, 3) - RJmat(1, 3) * RJmat(3, 1)
            RinvJ(2, 3) = RJmat(1, 3) * RJmat(2, 1) - RJmat(1, 1) * RJmat(2, 3)
            RinvJ(3, 1) = RJmat(2, 1) * RJmat(3, 2) - RJmat(2, 2) * RJmat(3, 1)
            RinvJ(3, 2) = RJmat(1, 2) * RJmat(3, 1) - RJmat(1, 1) * RJmat(3, 2)
            RinvJ(3, 3) = RJmat(1, 1) * RJmat(2, 2) - RJmat(1, 2) * RJmat(2, 1)
            '逆ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの配列に、面積分で使用するための値を保存する。
        End If

    End Sub

    Sub JacobElemC(ByVal ie As Integer, ByVal numg As Integer, _
        ByRef kh As Integer, ByRef dj As Double)
        '要素のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算（ｴﾗｰ停止なし）
        'ie･･･要素番号
        'numg･･･ｶﾞｳｽ点番号
        'kh･･･ｴﾗｰ有無（0:なし、-1:有り）
        'dj･･･ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの行列式値
        Dim i, ii, j, nmax As Integer
        Dim b1 As Double

        If KindElem = 0 Then
            nmax = 20
        End If
        kh = 0
        For i = 1 To 3
            For j = 1 To 3
                RinvJ(i, j) = 0
                RJmat(i, j) = 0
            Next j
        Next i
        XG(numg) = 0 : YG(numg) = 0 : ZG(numg) = 0
        UG(numg) = 0 : VG(numg) = 0 : WG(numg) = 0
        TG(numg) = 0
        For ii = 1 To nmax
            XG(numg) = XG(numg) + XE(ii) * Shape(ii)
            YG(numg) = YG(numg) + YE(ii) * Shape(ii)
            ZG(numg) = ZG(numg) + ZE(ii) * Shape(ii)
            UG(numg) = UG(numg) + UE(ii) * Shape(ii)
            VG(numg) = VG(numg) + VE(ii) * Shape(ii)
            WG(numg) = WG(numg) + WE(ii) * Shape(ii)
            TG(numg) = TG(numg) + TE(ii) * Shape(ii)
        Next ii

        For ii = 1 To nmax
            For j = 1 To 3
                Select Case j
                    Case 1
                        b1 = XE(ii)
                    Case 2
                        b1 = YE(ii)
                    Case 3
                        b1 = ZE(ii)
                End Select
                RJmat(1, j) = RJmat(1, j) + Ds(ii) * b1
                RJmat(2, j) = RJmat(2, j) + Dt(ii) * b1
                RJmat(3, j) = RJmat(3, j) + Du(ii) * b1
            Next j
        Next ii
        RinvJ(1, 1) = RJmat(2, 2) * RJmat(3, 3) - RJmat(2, 3) * RJmat(3, 2)
        RinvJ(1, 2) = RJmat(1, 3) * RJmat(3, 2) - RJmat(1, 2) * RJmat(3, 3)
        RinvJ(1, 3) = RJmat(1, 2) * RJmat(2, 3) - RJmat(1, 3) * RJmat(2, 2)
        RinvJ(2, 1) = RJmat(2, 3) * RJmat(3, 1) - RJmat(2, 1) * RJmat(3, 3)
        RinvJ(2, 2) = RJmat(1, 1) * RJmat(3, 3) - RJmat(1, 3) * RJmat(3, 1)
        RinvJ(2, 3) = RJmat(1, 3) * RJmat(2, 1) - RJmat(1, 1) * RJmat(2, 3)
        RinvJ(3, 1) = RJmat(2, 1) * RJmat(3, 2) - RJmat(2, 2) * RJmat(3, 1)
        RinvJ(3, 2) = RJmat(1, 2) * RJmat(3, 1) - RJmat(1, 1) * RJmat(3, 2)
        RinvJ(3, 3) = RJmat(1, 1) * RJmat(2, 2) - RJmat(1, 2) * RJmat(2, 1)
        b1 = RJmat(1, 1) * RinvJ(1, 1) + RJmat(1, 2) * RinvJ(2, 1)
        dj = b1 + RJmat(1, 3) * RinvJ(3, 1)

        If dj <= 0 Then
            kh = -1
            Exit Sub
        Else
            '本来の逆ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽは下記のようにdjで割った値である。
            For i = 1 To 3
                For j = 1 To 3
                    RinvJT(i, j) = RinvJ(i, j) / dj
                Next j
            Next i
            For ii = 1 To nmax
                For i = 1 To 3
                    b1 = RinvJ(i, 1) * Ds(ii) + RinvJ(i, 2) * Dt(ii)
                    b1 = (b1 + RinvJ(i, 3) * Du(ii)) / dj
                    Select Case i
                        Case 1
                            Dx(ii) = b1
                        Case 2
                            Dy(ii) = b1
                        Case 3
                            Dz(ii) = b1
                    End Select
                Next i
            Next ii
        End If

    End Sub

    Function KSideLine(ByVal ib As Integer, ByVal kb As Integer, _
    ByVal kk As Integer) As Integer
        '辺番号の照合
        'ib･･･要素番号
        'kb･･･面番号
        'kk･･･面の辺番号
        Dim ksb As Integer

        If KindElem = 0 Then    '六面体
            Select Case kb
                Case 1
                    ksb = kk
                Case 2
                    ksb = 13 - kk
                Case 3
                    Select Case kk
                        Case 1
                            ksb = 5
                        Case 2
                            ksb = 9
                        Case 3
                            ksb = 6
                        Case 4
                            ksb = 1
                    End Select
                Case 4
                    Select Case kk
                        Case 1
                            ksb = 7
                        Case 2
                            ksb = 11
                        Case 3
                            ksb = 8
                        Case 4
                            ksb = 3
                    End Select
                Case 5
                    Select Case kk
                        Case 1
                            ksb = 6
                        Case 2
                            ksb = 10
                        Case 3
                            ksb = 7
                        Case 4
                            ksb = 2
                    End Select
                Case 6
                    Select Case kk
                        Case 1
                            ksb = 8
                        Case 2
                            ksb = 12
                        Case 3
                            ksb = 5
                        Case 4
                            ksb = 4
                    End Select
            End Select
            KSideLine = 12 * (ib - 1) + ksb
        End If

    End Function

    Sub MiniSort2(ByVal kbuf() As Integer, ByRef nsort() As Integer, ByVal work() As Single, _
    ByVal xbuf() As Single, ByVal ybuf() As Single, ByVal zbuf() As Single)
        'ﾐﾆｿｰﾄ2
        '同じ変数値の場合に、第2座標値でソート
        'kind･･･主ｿｰﾄの座標軸番号（1:X軸、2:Y軸）
        'nsort()･･･元の変数番号のｿｰﾄ順番号
        'work()･･･ｿｰﾄ変数値
        'xbuf(),ybuf()･･･節点の座標
        Dim i, j, jb, jx, k, k1, k2, kx, nb, n1, n2, nx As Integer
        Dim b1 As Single
        Dim nsbuf(10), nsort2(10), nsort3(10) As Integer
        Dim work2(10), work3(10) As Single

        i = 1
        Do
            If work(i) = work(i + 1) Then
                b1 = work(i)
                n1 = i
                For j = i + 1 To Npoint
                    If b1 = work(j) Then
                        n2 = j
                    Else
                        Exit For
                    End If
                Next j
                nx = n2 - n1 + 1
                ReDim nsort2(nx), work2(nx)
                jb = 0
                For j = n1 To n2
                    jb = jb + 1
                    nsort2(jb) = jb
                    nb = nsort(j)
                    Select Case kbuf(2)
                        Case 1
                            work2(jb) = xbuf(nb)
                        Case 2
                            work2(jb) = ybuf(nb)
                        Case 3
                            work2(jb) = zbuf(nb)
                    End Select
                Next j
                Call ShellSortStd(nx, nsort2, work2)    'ｼｪｯﾙｿｰﾄ基本

                k = 1
                Do
                    If work2(k) = work2(k + 1) Then
                        b1 = work2(k)
                        k1 = k
                        For j = k + 1 To nx
                            If b1 = work2(j) Then
                                k2 = j
                            Else
                                Exit For
                            End If
                        Next j
                        kx = k2 - k1 + 1
                        ReDim nsort3(kx), work3(kx), nsbuf(kx)
                        jb = 0
                        For j = k1 To k2
                            jb = jb + 1
                            nsbuf(jb) = nsort2(j)
                            nsort3(jb) = jb
                            nb = nsort(n1 + nsort2(j) - 1)
                            Select Case kbuf(3)
                                Case 1
                                    work3(jb) = xbuf(nb)
                                Case 2
                                    work3(jb) = ybuf(nb)
                                Case 3
                                    work3(jb) = zbuf(nb)
                            End Select
                        Next j
                        Call ShellSortStd(kx, nsort3, work3)    'ｼｪｯﾙｿｰﾄ基本
                        For j = 1 To kx
                            jb = k1 + j - 1
                            nsort2(jb) = nsbuf(nsort3(j))
                        Next j
                        k = k2 + 1
                    Else
                        k = k + 1
                    End If
                Loop While k < nx

                For j = 1 To nx
                    jb = n1 + nsort2(j) - 1
                    nsort2(j) = nsort(jb)
                Next j
                For j = 1 To nx
                    jx = n1 + j - 1
                    nsort(jx) = nsort2(j)
                Next j
                i = n2 + 1
            Else
                i = i + 1
            End If
        Loop While i < Npoint

    End Sub

    Sub ModelTrans(ByVal nb As Integer, ByRef xbuf() As Single, _
    ByRef ybuf() As Single, ByRef zbuf() As Single)
        'ﾓﾃﾞﾙ節点座標のｱﾌｨﾝ変換
        'nb･･･節点数
        'xbuf(),ybuf(),zbuf()･･･節点のX,Y,Z座標
        Dim ip, j, k As Integer
        Dim b1 As Single
        Dim buf(3) As Single
        For ip = 1 To nb
            buf(1) = xbuf(ip)
            buf(2) = ybuf(ip)
            buf(3) = zbuf(ip)
            For k = 1 To 3
                b1 = 0
                For j = 1 To 3
                    b1 = b1 + TransMat(k, j) * buf(j)
                Next j
                b1 = b1 + TransMat(k, 4)
                Select Case k
                    Case 1
                        xbuf(ip) = b1
                    Case 2
                        ybuf(ip) = b1
                    Case 3
                        zbuf(ip) = b1
                End Select
            Next k
        Next ip

    End Sub

    Sub MouseCalc()
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        Dim j As Integer
        Dim xa, xb, ya, yb As Single
        For j = 1 To 2
            xa = Xmouse(j)
            ya = Ymouse(j)
            If Cfix < 1 Then
                xb = (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
            Else
                xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale + YaxisMin
            End If
            Xmouse(j) = xb
            Ymouse(j) = yb
        Next j

    End Sub

    Sub MousePointSearch(ByRef kp As Integer, ByVal xb As Single, ByVal yb As Single)
        'ﾏｳｽ点の節点の探索
        'kp･･･ﾏｳｽ点の節点番号
        'xb,yb･･･ﾏｳｽ点の作画座標位置
        Dim ie, k, kb, kk, kmax, nb As Integer
        Dim b1, b2, b3, rlen2, x2, y2 As Single
        Dim kbuf(Npoint) As Integer

        If KindElem = 0 Then
            kmax = 8
        End If
        kp = 0
        For nb = NSdraw To 1 Step -1
            ie = KEorder(nb)
            kb = KSorder(nb)
            If KindElem = 0 Then
                Call SurfaceSet(ie, kb, KsurfPoint)  '面の設定
            End If
            For k = 1 To kmax
                kk = KsurfPoint(k)
                If kbuf(kk) = 0 Then
                    kbuf(kk) = 1
                    x2 = Xscreen(kk)
                    y2 = Yscreen(kk)
                    Call XYCalc(x2, y2)     'X,Yの作画座標計算
                    b1 = x2 - xb
                    b2 = y2 - yb
                    b3 = b1 * b1 + b2 * b2
                    If kp = 0 Or b3 < rlen2 Then
                        kp = kk
                        rlen2 = b3
                    End If
                End If
            Next k
        Next nb

    End Sub

    Sub OptFileIn()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\FL3DMenu.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            For i = 1 To 3
                Koption(i) = Val(sr.ReadLine)
            Next i
            Kscreen = Val(sr.ReadLine)
            KcomSide = Val(sr.ReadLine)
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub OptFileOut()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\FL3DMenu.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 3
            sw.WriteLine(Koption(i))
        Next i
        sw.WriteLine(Kscreen)
        sw.WriteLine(KcomSide)
        sw.Close()

    End Sub

    Sub OuterReset(ByVal k1 As Integer, ByVal k2 As Integer)
        '外郭面の再設定
        'k1,k2･･･開始・終了要素番号
        Dim ie, k, kmax As Integer

        If KindElem = 0 Then
            kmax = 6
        End If
        For ie = k1 To k2
            For k = 1 To kmax
                If KSelem(ie, k) < 0 Then
                    KSelem(ie, k) = 0
                End If
            Next k
        Next ie
    End Sub

    Sub OutSideCheck()
        '外周面のﾁｪｯｸ
        Dim ib, ic, ie, ik, ik1, j, jj, jmax, jx, k, kk, kmax As Integer
        Dim kbuf(8) As Integer
        ReDim KEInside(Nelem, 6)

        If KindElem = 0 Then
            jmax = 8
            kmax = 6
        End If
        For ie = 1 To Nelem - 1
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If KEInside(ie, k) = 0 Then
                        If KindElem = 0 Then
                            Call SurfaceSet(ie, k, KsurfPoint)  '面の設定
                        End If
                        ik = 0
                        For ib = ie + 1 To Nelem
                            If KdrawElem(ib) > 0 Then
                                For kk = 1 To kmax
                                    If KEInside(ib, kk) = 0 Then
                                        If KindElem = 0 Then
                                            Call SurfaceSet(ib, kk, kbuf)   '面の設定
                                        End If
                                        ik1 = KsurfPoint(1)
                                        ic = 0
                                        For j = 1 To jmax
                                            If ik1 = kbuf(j) Then
                                                jx = j
                                                For jj = 2 To jmax
                                                    jx = jx - 1
                                                    If jx < 1 Then
                                                        jx = jmax
                                                    End If
                                                    If KsurfPoint(jj) <> kbuf(jx) Then
                                                        ic = 1
                                                        Exit For
                                                    End If
                                                Next jj
                                                If ic = 0 Then
                                                    ik = 1
                                                    KEInside(ie, k) = 1
                                                    KEInside(ib, kk) = 1
                                                End If
                                                Exit For
                                            End If
                                        Next j
                                        If ik = 1 Then
                                            Exit For
                                        End If
                                    End If
                                Next kk
                                If ik = 1 Then
                                    Exit For
                                End If
                            End If
                        Next ib
                    End If
                Next k
            End If
        Next ie

    End Sub

    Sub PaintExe(ByRef gr As Graphics, ByVal ncount As Integer, _
        ByVal colorx As Color, ByVal kind As Integer)
        '塗りつぶし実行
        'gr･･･ｸﾞﾗﾌｨｯｸｽ･ｵﾌﾞｼﾞｪｸﾄ名
        'ncount･･･頂点数
        'colorx･･･境界および塗りつぶしの色
        'kind･･･境界作画の有無（0:なし、1:有り）
        Dim i As Integer
        Dim xydraw(ncount - 1) As Point
        Dim myBrush As New SolidBrush(colorx)

        For i = 1 To ncount
            xydraw(i - 1).X = Xdraw(i) + 0.001
            xydraw(i - 1).Y = Ydraw(i) + 0.001
        Next i
        gr.FillPolygon(myBrush, xydraw)

        If kind > 0 Then
            gr.DrawPolygon(Pens.Black, xydraw)
        End If

    End Sub

    Sub ParaOutS(ftemp$)
        '結果ﾊﾟﾗﾒｰﾀｰ出力S
        'ftemp$･･･出力ﾌｧｲﾙ名

        Dim aa$
        Dim i, j As Integer

        Dim sw As New IO.StreamWriter(ftemp$)
        sw.WriteLine("パラメーター評価セット")
        aa$ = "組数" + ",  " + Str(NparaSet)
        sw.WriteLine(aa$)
        aa$ = "＜組番号、追加係数、要素数、Re、収束回数評価、流出入評価、速度変化評価＞"
        sw.WriteLine(aa$)
        For i = 1 To NparaSet
            aa$ = Str(i)
            For j = 1 To 6
                aa$ = aa$ + ",  " + Str(ParaSet(i, j))
            Next j
            sw.WriteLine(aa$)
        Next i

        aa$ = "AICalcの結果" + ",  " + Str(KAICalc)
        sw.WriteLine(aa$)
        sw.Close()

    End Sub

    Sub PDiffVelCalc(ByVal ik As Integer)
        '速度偏微分の計算
        'ik･･･繰り返し回数
        Dim i, i2, icount, ie, ig, ii, j, jg, jmax As Integer
        Dim k, kb, kg, kk, m, nmax, numg As Integer
        Dim b1, dj, s, t, u As Double
        Dim tfb, tfx, tfxx As Double
        Dim vsum(Npoint) As Single
        Dim buf(9), pdbuf(20, 9), pdgbuf(27, 9) As Double

        For i = 1 To Npoint
            For j = 1 To 9
                PDVelXX(i, j) = PDVelX(i, j)
                PDVelX(i, j) = PDVel(i, j)
                PDVel(i, j) = 0
            Next j
        Next i

        If KindElem = 0 Then
            jmax = 27
            nmax = 20
        End If
        For i = 1 To Npoint
            For j = 1 To 9
                PDVel(i, j) = 0
            Next j
        Next i

        If ik > 1 Then
            icount = 0
            For ie = 1 To Nelem
                For i = 1 To nmax
                    i2 = NNelem(ie, i)
                    k = Kcomp(i2)
                    XE(i) = Xdata(i2)
                    YE(i) = Ydata(i2)
                    ZE(i) = Zdata(i2)
                    kk = Ndim * (k - 1) + 1
                    For m = 0 To 2
                        kb = Lcomp(kk + m)
                        Select Case m
                            Case 0
                                If kb = 0 Then
                                    UE(i) = 0
                                Else
                                    UE(i) = Tff(kb)
                                End If
                            Case 1
                                If kb = 0 Then
                                    VE(i) = 0
                                Else
                                    VE(i) = Tff(kb)
                                End If
                            Case 2
                                If kb = 0 Then
                                    WE(i) = 0
                                Else
                                    WE(i) = Tff(kb)
                                End If
                        End Select
                    Next m
                Next i

                If KindElem = 0 Then
                    numg = 0
                    For ig = 1 To Ngaus
                        s = PosGaus(ig)
                        For jg = 1 To Ngaus
                            t = PosGaus(jg)
                            For kg = 1 To Ngaus
                                u = PosGaus(kg)
                                numg = numg + 1
                                Call ShapeFunc(1, s, t, u)    '形状関数の計算
                                Call JacobElem(ie, 1, numg, dj)    '要素のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算

                                For i = 1 To 9
                                    buf(i) = 0
                                Next i
                                For i = 1 To 20
                                    buf(1) = buf(1) + UE(i) * Dx(i)
                                    buf(2) = buf(2) + UE(i) * Dy(i)
                                    buf(3) = buf(3) + UE(i) * Dz(i)
                                    buf(4) = buf(4) + VE(i) * Dx(i)
                                    buf(5) = buf(5) + VE(i) * Dy(i)
                                    buf(6) = buf(6) + VE(i) * Dz(i)
                                    buf(7) = buf(7) + WE(i) * Dx(i)
                                    buf(8) = buf(8) + WE(i) * Dy(i)
                                    buf(9) = buf(9) + WE(i) * Dz(i)
                                Next i
                                For i = 1 To 9
                                    pdgbuf(numg, i) = buf(i)
                                Next i
                            Next kg
                        Next jg
                    Next ig
                End If

                For i = 1 To nmax
                    For k = 1 To 9
                        pdbuf(i, k) = 0
                    Next k
                    For j = 1 To jmax
                        b1 = Tr(i, j)
                        For k = 1 To 9
                            pdbuf(i, k) = pdbuf(i, k) + b1 * pdgbuf(j, k)
                        Next k
                    Next j
                Next i
                For ii = 1 To nmax
                    i = Kcomp(NNelem(ie, ii))
                    vsum(i) = vsum(i) + VolE(ie)
                    For k = 1 To 9
                        PDVel(i, k) = PDVel(i, k) + pdbuf(ii, k) * VolE(ie)
                    Next k
                Next ii
                If icount < 99 Then
                    icount = icount + 1
                Else
                    Application.DoEvents()
                    icount = 0
                End If
            Next ie

            For i = 1 To Npoint
                For k = 1 To 9
                    PDVel(i, k) = PDVel(i, k) / vsum(i)
                Next k
            Next i

            '緩和処理
            If ik = 2 Then
                For i = 1 To Npoint
                    For k = 1 To 9
                        tfb = PDVel(i, k)
                        tfx = PDVelX(i, k)
                        'Call RelaxCalc2B(tfb, tfx) '緩和計算
                        tfb = RelaxCalFS(tfb, tfx) '緩和計算
                        PDVel(i, k) = tfb
                    Next k
                Next i
            Else
                For i = 1 To Npoint
                    For k = 1 To 9
                        tfb = PDVel(i, k)
                        tfx = PDVelX(i, k)
                        tfxx = PDVelXX(i, k)
                        Call RelaxCalcB(tfb, tfx, tfxx) '緩和計算
                        PDVel(i, k) = tfb
                    Next k
                Next i
            End If
        End If

    End Sub

    Sub PDiffVelCalcT()
        '非定常の速度偏微分の計算
        Dim i, i2, icount, ie, ig, ii, j, jg, jmax, k, kb, kg, kk, m, nmax, numg As Integer
        Dim b1, dj, s, t, u As Double
        Dim vsum(Npoint) As Single
        Dim buf(9), pdbuf(20, 9), pdgbuf(27, 9) As Double

        If KindElem = 0 Then
            jmax = 27
            nmax = 20
        End If
        For i = 1 To Npoint
            For j = 1 To 9
                PDVel(i, j) = 0
            Next j
        Next i

        icount = 0
        For ie = 1 To Nelem
            For i = 1 To nmax
                i2 = NNelem(ie, i)
                k = Kcomp(i2)
                XE(i) = Xdata(i2)
                YE(i) = Ydata(i2)
                ZE(i) = Zdata(i2)
                kk = Ndim * (k - 1) + 1
                For m = 0 To 3
                    kb = Lcomp(kk + m)
                    b1 = 0.5 * (Tff(kb) + Tffx(kb))
                    Select Case m
                        Case 0
                            If kb = 0 Then
                                UE(i) = 0
                                UEb(i) = 0
                            Else
                                UE(i) = b1
                                UEb(i) = Tffx(kb)
                            End If
                        Case 1
                            If kb = 0 Then
                                VE(i) = 0
                                VEb(i) = 0
                            Else
                                VE(i) = b1
                                VEb(i) = Tffx(kb)
                            End If
                        Case 2
                            If kb = 0 Then
                                WE(i) = 0
                                WEb(i) = 0
                            Else
                                WE(i) = b1
                                WEb(i) = Tffx(kb)
                            End If
                    End Select
                Next m
            Next i

            If KindElem = 0 Then
                numg = 0
                For ig = 1 To Ngaus
                    s = PosGaus(ig)
                    For jg = 1 To Ngaus
                        t = PosGaus(jg)
                        For kg = 1 To Ngaus
                            u = PosGaus(kg)
                            numg = numg + 1
                            Call ShapeFunc(1, s, t, u)    '形状関数の計算
                            Call JacobElem(ie, 1, numg, dj)    '要素のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算

                            For i = 1 To 9
                                buf(i) = 0
                            Next i
                            For i = 1 To 20
                                buf(1) = buf(1) + UE(i) * Dx(i)
                                buf(2) = buf(2) + UE(i) * Dy(i)
                                buf(3) = buf(3) + UE(i) * Dz(i)
                                buf(4) = buf(4) + VE(i) * Dx(i)
                                buf(5) = buf(5) + VE(i) * Dy(i)
                                buf(6) = buf(6) + VE(i) * Dz(i)
                                buf(7) = buf(7) + WE(i) * Dx(i)
                                buf(8) = buf(8) + WE(i) * Dy(i)
                                buf(9) = buf(9) + WE(i) * Dz(i)
                            Next i
                            For i = 1 To 9
                                pdgbuf(numg, i) = buf(i)
                            Next i
                        Next kg
                    Next jg
                Next ig
            End If

            For i = 1 To nmax
                For k = 1 To 9
                    pdbuf(i, k) = 0
                Next k
                For j = 1 To jmax
                    b1 = Tr(i, j)
                    For k = 1 To 9
                        pdbuf(i, k) = pdbuf(i, k) + b1 * pdgbuf(j, k)
                    Next k
                Next j
            Next i
            For ii = 1 To nmax
                i = Kcomp(NNelem(ie, ii))
                vsum(i) = vsum(i) + VolE(ie)
                For k = 1 To 9
                    PDVel(i, k) = PDVel(i, k) + pdbuf(ii, k) * VolE(ie)
                Next k
            Next ii
            If icount < 99 Then
                icount = icount + 1
            Else
                Application.DoEvents()
                icount = 0
            End If
        Next ie

        For i = 1 To Npoint
            For k = 1 To 9
                PDVel(i, k) = PDVel(i, k) / vsum(i)
            Next k
        Next i

    End Sub

    Sub PerspecCalc(ByVal k1 As Integer, ByVal k2 As Integer, ByVal rlenscreen As Single, _
    ByRef xvbuf() As Single, ByRef yvbuf() As Single, ByRef zvbuf() As Single, _
    ByRef xsbuf() As Single, ByRef ysbuf() As Single)
        'ｽｸﾘｰﾝへの透視座標の計算
        'k1,k2･･･要素の開始、終了番号
        'rlenscreen･･･透視座標原点からｽｸﾘｰﾝまでの距離
        'xvbuf(),yvbuf(),zvbuf()･･･視線座標のX,Y,Z
        'xsbuf(),ysbuf()･･･ｽｸﾘｰﾝ座標のX,Y
        Dim ib, ie, j, nmax As Integer
        Dim b1 As Single
        Dim kbuf(Npoint) As Integer

        If KindElem = 0 Then
            nmax = 20
        End If
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    ib = NNelem(ie, j)
                    If kbuf(ib) = 0 Then
                        b1 = rlenscreen / zvbuf(ib)
                        xsbuf(ib) = b1 * xvbuf(ib)
                        ysbuf(ib) = b1 * (yvbuf(ib) - Eyeh)
                        kbuf(ib) = 1
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub PlaneCutElem()
        '面切断の要素選択
        Dim ic, ie, j, k, nmax As Integer
        Dim b1, std As Single

        If KindElem = 0 Then
            nmax = 20
        End If
        If KE3DCut(2) > 0 Then
            std = VE3DCut(1) - VE3DCut(2)
        Else
            std = VE3DCut(1) + VE3DCut(2)
        End If

        For ie = 1 To Nelem
            If KdrawElem(ie) > 0 Then
                ic = 0
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    Select Case KE3DCut(1)
                        Case 1
                            b1 = Xdata(k)
                        Case 2
                            b1 = Ydata(k)
                        Case 3
                            b1 = Zdata(k)
                    End Select
                    If KE3DCut(2) > 0 Then
                        If b1 < std Then
                            ic = 1
                        End If
                    Else
                        If b1 > std Then
                            ic = 1
                        End If
                    End If
                    If ic > 0 Then
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub PnumImproveNew(ByRef mb As Integer, ByVal mbx As Integer, _
    ByVal xbuf() As Single, ByVal ybuf() As Single, ByVal zbuf() As Single)
        '節点順序の改良
        'mb･･･改良後のﾊﾞﾝﾄﾞ幅
        'mbx･･･改良前のﾊﾞﾝﾄﾞ幅
        'xbuf(),ybuf(),zbuf()･･･節点の座標
        Dim i, ib, ic, ie, k, k1, k2, mbb As Integer
        Dim rlenb As Single
        Dim kbuf(3), kbufb(3), kcbuf(Npoint) As Integer
        Dim rlbuf(3) As Single

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie
        Call AxisCheck(1, Nelem, rlenb)     'X、Y、Z軸の最小・最大値のﾁｪｯｸ

        mbb = 0
        For k = 1 To 3
            kbuf(1) = k
            Select Case k
                Case 1
                    k1 = 2
                    k2 = 3
                Case 2
                    k1 = 3
                    k2 = 1
                Case 3
                    k1 = 1
                    k2 = 2
            End Select
            rlbuf(k1) = AxisMax(k1) - AxisMin(k1)
            rlbuf(k2) = AxisMax(k2) - AxisMin(k2)
            If rlbuf(k1) > rlbuf(k2) Then
                kbuf(2) = k1
                kbuf(3) = k2
            Else
                kbuf(2) = k2
                kbuf(3) = k1
            End If

            Call ShellSort2(kbuf, kcbuf, xbuf, ybuf, zbuf)    'ｼｪﾙｿｰﾄ2
            Call BandWidth2(mb, kcbuf)      'ﾊﾞﾝﾄﾞ幅の計算
            ic = 0
            If k = 1 Or mb < mbb Then
                ic = 1
            End If
            If ic > 0 Then
                For i = 1 To 3
                    kbufb(i) = kbuf(i)
                Next i
                mbb = mb
            End If
        Next k

        If mbb < mbx Then
            For i = 1 To 3
                kbuf(i) = kbufb(i)
            Next i
            Call ShellSort2(kbuf, kcbuf, xbuf, ybuf, zbuf)    'ｼｪﾙｿｰﾄ2
            Call BandWidth2(mb, kcbuf)      'ﾊﾞﾝﾄﾞ幅の計算
            For i = 1 To Npoint
                Kcomp(i) = kcbuf(i)
            Next i
        Else
            mb = mbx
        End If

        For i = 1 To Npoint
            ib = Kcomp(i)
            KRcomp(ib) = i
        Next i

    End Sub

    Sub PointSearch(ByRef nd As Integer)
        '矩形領域の節点の探索
        'nb･･･適合節点の数
        Dim i, ie, k, kb, kk, nb, nmax As Integer
        Dim b1, b2 As Single

        If KindElem = 0 Then
            nmax = 8
            ReDim KinnerLine(12 * Nelem) '内部辺のﾌﾗｯｸﾞ
        End If
        For i = 1 To Npoint
            KAppPoint(i) = 0
        Next i
        nd = 0
        For nb = 1 To NSdraw
            ie = KEorder(nb)
            kb = KSorder(nb)
            If KindElem = 0 Then
                Call SurfaceSet(ie, kb, KsurfPoint)  '面の設定
            End If
            For k = 1 To nmax
                kk = KsurfPoint(k)
                If KAppPoint(kk) = 0 Then
                    b1 = Xscreen(kk) - Xmouse(1)
                    b2 = Xscreen(kk) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        b1 = Yscreen(kk) - Ymouse(2)
                        b2 = Yscreen(kk) - Ymouse(1)
                        If b1 >= 0 And b2 <= 0 Then
                            KAppPoint(kk) = 1
                            nd = nd + 1
                        End If
                    End If
                End If
            Next k
        Next nb

    End Sub

    Sub RangeCheck(ByRef kh As Integer, ByVal epsL As Single, ByRef xrange() As Single,
        ByRef xrange2() As Single, ByRef yrange() As Single, ByRef yrange2() As Single)
        '範囲ﾁｪｯｸ
        'kh･･･判定区分（0:判定なし、1:別範囲）
        'epsL･･･微小誤差長さ
        kh = 0
        If xrange(2) < xrange2(1) + epsL Or xrange2(2) < xrange(1) + epsL Then
            kh = 1
        ElseIf yrange(2) < yrange2(1) + epsL Or yrange2(2) < yrange(1) + epsL Then
            kh = 1
        End If
    End Sub

    Sub RelaxCalc(ByRef tfb As Double, ByVal tfx As Double, _
        ByVal tfxx As Double)
        '緩和計算
        'tfb･･･今回値
        'tfx･･･前回値
        'tfxx･･･前々回値
        Dim b1, b3, b4 As Double
        b1 = tfb - tfx
        b3 = Math.Abs(b1)
        b4 = Math.Abs(tfx - tfxx)
        If b3 > b4 Then
            If b4 = 0 Then
                b4 = 0.2 * b3
            End If
            tfb = tfx + b1 * b4 / b3
        End If
        tfb = (tfb + 3 * tfx + tfxx) / 5
        'tfb = (tfb + 2 * tfx + 3 * tfxx) / 6

    End Sub

    Sub RelaxCalc2B(ByRef tfb As Double, ByVal tfx As Double)
        '繰返し2回目の緩和計算
        'tfb･･･今回値
        'tfx･･･前回値
        Dim b1, b2, b3, b4, c1 As Double

        c1 = 0.1
        b1 = tfb - tfx
        b3 = Math.Abs(b1)
        b4 = Math.Abs(tfx)
        If b3 > c1 * b4 Then
            If b4 = 0 Then
                b4 = 0.2 * b3
            End If
            b2 = b3 / b4
            If b2 < 1 Then
                c1 = 0.2
            ElseIf b2 < 1.424 Then
                b3 = b2 - 1
                c1 = 0.2 - b3 * b3
            ElseIf b2 < 3 Then
                c1 = 0.002
            Else
                c1 = 0.000001
            End If
            tfb = tfx + b1 * c1
        Else
            tfb = 0.5 * (tfb + tfx)
        End If

    End Sub

    Sub RelaxCalcB(ByRef tfb As Double, ByVal tfx As Double, _
        ByVal tfxx As Double)
        '緩和計算
        'tfb･･･今回値
        'tfx･･･前回値
        'tfxx･･･前々回値
        Dim b1, b2, b3, b4, c1 As Double

        b1 = tfb - tfx
        b3 = Math.Abs(b1)
        b4 = Math.Abs(tfx - tfxx)
        If b3 > 0 Then
            If b4 = 0 Then
                b4 = 0.2 * b3
            End If
            If b4 > 0 Then
                b2 = b3 / b4
                If b2 < 1 Then
                    c1 = 0.5
                ElseIf b2 < 1.703 Then
                    b3 = b2 - 1
                    c1 = 0.5 - b3 * b3
                ElseIf b2 < 3 Then
                    c1 = 0.005
                Else
                    c1 = 0.000001
                End If
                tfb = tfx + b1 * c1
            End If
        End If
        tfb = (tfb + 3 * tfx + tfxx) / 5

    End Sub

    Sub RelaxCalcD(ByRef tfb As Double, ByVal tfx As Double,
        ByVal tfxx As Double)
        '緩和計算
        'tfb･･･今回値
        'tfx･･･前回値
        'tfxx･･･前々回値
        Dim b1, b3, b4 As Double

        GoTo Skip10
        b1 = tfb - tfx
        b3 = Math.Abs(b1)
        b4 = Math.Abs(tfx - tfxx)
        If b3 > b4 Then
            If b4 = 0 Then
                b4 = 0.2 * b3
            End If
            tfb = tfx + b1 * b4 / b3
        End If
Skip10:

        tfb = (tfb + 3 * tfx + tfxx) / 5
        'tfb = (tfb + tfx + tfxx) / 3

    End Sub

    Function RelaxCalF1(ByVal tfb As Double, ByVal tfx As Double) As Double
        '繰返し2回目の緩和計算F1
        'tfb･･･今回値
        'tfx･･･前回値
        Dim tfnew As Double

        tfnew = 0.5 * (tfb + tfx)
        Return tfnew

    End Function

    Function RelaxCalFS(ByVal tfb As Double, ByVal tfx As Double) As Double
        '繰返し2回目の緩和計算
        'tfb･･･今回値
        'tfx･･･前回値
        Dim b1, b2 As Single
        Dim tfnew As Double

        b1 = 0.000001

        'b1 = 0.001
        b2 = 1 - b1
        tfnew = b1 * tfb + b2 * tfx
        Return tfnew

    End Function

    Sub RotateCalc(ByVal kind As Integer, ByRef theta As Single)
        '回転角の計算
        'kind･･･象限番号
        'theta･･･回転角（左ねじ正、°）
        Dim ix, iy, kb As Integer
        Dim b1 As Single

        ix = Xmouse(2) - Xmouse(1)
        iy = Ymouse(2) - Ymouse(1)
        Select Case kind
            Case 1
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                End If
            Case 2
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                Else
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                End If
            Case 3
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                End If
            Case 4
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                Else
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                End If
        End Select
        b1 = Math.Sqrt(ix * ix + iy * iy)
        theta = kb * 3.0 * b1 / 50.0

    End Sub

    Sub RoundPrep(ByRef kind As Integer, ByRef nb As Integer, ByVal zb As Single)
        '丸め準備
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･丸め基準値
        Dim i, ib As Integer
        Dim b2 As Single

        b2 = zb
        nb = 0
        i = Int(b2)
        If i > 0 Then
            kind = 0
            Do
                ib = CInt(b2)
                If ib < 10 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 0.1
                End If
            Loop
        Else
            kind = 1
            Do
                ib = CInt(b2)
                If ib > 0 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 10
                End If
            Loop
        End If

    End Sub

    Sub RoundTreat(ByVal kind As Integer, ByVal nb As Integer, ByRef zb As Single)
        '丸め処理
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･対象変数
        Dim ib As Integer
        Dim b1 As Single

        If kind = 0 Then
            b1 = zb * 10 ^ (-nb)
            ib = CInt(b1)
            zb = ib * 10 ^ nb
        Else
            b1 = zb * 10 ^ nb
            ib = CInt(b1)
            zb = ib * 10 ^ (-nb)
        End If

    End Sub

    Sub ShapeFunc(ByVal kind As Integer, ByVal s As Double, ByVal t As Double, _
    ByVal u As Double)
        '形状関数の計算
        'kind･･･計算区分（0:基本、1:偏微分込み）
        's･･･要素のξ座標
        't･･･要素のη座標
        'u･･･要素のζ座標
        Dim s1, s1m, s2m, t1, t1m, t2m, u1, u1m, u2m As Double
        s1 = 1 + s
        s1m = 1 - s
        s2m = 1 - s * s
        t1 = 1 + t
        t1m = 1 - t
        t2m = 1 - t * t
        u1 = 1 + u
        u1m = 1 - u
        u2m = 1 - u * u
        Shape(1) = s1m * t1m * u1 * (-s - t + u - 2) / 8
        Shape(2) = s2m * t1m * u1 / 4
        Shape(3) = s1 * t1m * u1 * (s - t + u - 2) / 8
        Shape(4) = s1 * t2m * u1 / 4
        Shape(5) = s1 * t1 * u1 * (s + t + u - 2) / 8
        Shape(6) = s2m * t1 * u1 / 4
        Shape(7) = s1m * t1 * u1 * (-s + t + u - 2) / 8
        Shape(8) = s1m * t2m * u1 / 4
        Shape(9) = s1m * t1m * u2m / 4
        Shape(10) = s1 * t1m * u2m / 4
        Shape(11) = s1 * t1 * u2m / 4
        Shape(12) = s1m * t1 * u2m / 4
        Shape(13) = s1m * t1m * u1m * (-s - t - u - 2) / 8
        Shape(14) = s2m * t1m * u1m / 4
        Shape(15) = s1 * t1m * u1m * (s - t - u - 2) / 8
        Shape(16) = s1 * t2m * u1m / 4
        Shape(17) = s1 * t1 * u1m * (s + t - u - 2) / 8
        Shape(18) = s2m * t1 * u1m / 4
        Shape(19) = s1m * t1 * u1m * (-s + t - u - 2) / 8
        Shape(20) = s1m * t2m * u1m / 4

        If kind > 0 Then
            Ds(1) = t1m * u1 * (s + t - u + 1 + s) / 8
            Ds(2) = -t1m * u1 * s / 2
            Ds(3) = t1m * u1 * (s - t + u - 1 + s) / 8
            Ds(4) = t2m * u1 / 4
            Ds(5) = t1 * u1 * (s + t + u - 1 + s) / 8
            Ds(6) = -t1 * u1 * s / 2
            Ds(7) = t1 * u1 * (s - t - u + 1 + s) / 8
            Ds(8) = -t2m * u1 / 4
            Ds(9) = -t1m * u2m / 4
            Ds(10) = t1m * u2m / 4
            Ds(11) = t1 * u2m / 4
            Ds(12) = -t1 * u2m / 4
            Ds(13) = t1m * u1m * (s + t + u + 1 + s) / 8
            Ds(14) = -t1m * u1m * s / 2
            Ds(15) = t1m * u1m * (s - t - u - 1 + s) / 8
            Ds(16) = t2m * u1m / 4
            Ds(17) = t1 * u1m * (s + t - u - 1 + s) / 8
            Ds(18) = -t1 * u1m * s / 2
            Ds(19) = t1 * u1m * (s - t + u + 1 + s) / 8
            Ds(20) = -t2m * u1m / 4
            Dt(1) = s1m * u1 * (s + t - u + 1 + t) / 8
            Dt(2) = -s2m * u1 / 4
            Dt(3) = s1 * u1 * (-s + t - u + 1 + t) / 8
            Dt(4) = -s1 * u1 * t / 2
            Dt(5) = s1 * u1 * (s + t + u - 1 + t) / 8
            Dt(6) = s2m * u1 / 4
            Dt(7) = s1m * u1 * (-s + t + u - 1 + t) / 8
            Dt(8) = -s1m * u1 * t / 2
            Dt(9) = -s1m * u2m / 4
            Dt(10) = -s1 * u2m / 4
            Dt(11) = s1 * u2m / 4
            Dt(12) = s1m * u2m / 4
            Dt(13) = s1m * u1m * (s + t + u + 1 + t) / 8
            Dt(14) = -s2m * u1m / 4
            Dt(15) = s1 * u1m * (-s + t + u + 1 + t) / 8
            Dt(16) = -s1 * u1m * t / 2
            Dt(17) = s1 * u1m * (s + t - u - 1 + t) / 8
            Dt(18) = s2m * u1m / 4
            Dt(19) = s1m * u1m * (-s + t - u - 1 + t) / 8
            Dt(20) = -s1m * u1m * t / 2
            Du(1) = s1m * t1m * (-s - t + u - 1 + u) / 8
            Du(2) = s2m * t1m / 4
            Du(3) = s1 * t1m * (s - t + u - 1 + u) / 8
            Du(4) = s1 * t2m / 4
            Du(5) = s1 * t1 * (s + t + u - 1 + u) / 8
            Du(6) = s2m * t1 / 4
            Du(7) = s1m * t1 * (-s + t + u - 1 + u) / 8
            Du(8) = s1m * t2m / 4
            Du(9) = -s1m * t1m * u / 2
            Du(10) = -s1 * t1m * u / 2
            Du(11) = -s1 * t1 * u / 2
            Du(12) = -s1m * t1 * u / 2
            Du(13) = s1m * t1m * (s + t + u + 1 + u) / 8
            Du(14) = -s2m * t1m / 4
            Du(15) = s1 * t1m * (-s + t + u + 1 + u) / 8
            Du(16) = -s1 * t2m / 4
            Du(17) = s1 * t1 * (-s - t + u + 1 + u) / 8
            Du(18) = -s2m * t1 / 4
            Du(19) = s1m * t1 * (s - t + u + 1 + u) / 8
            Du(20) = -s1m * t2m / 4
        End If
    End Sub

    Sub ShapeFunc2(ByVal s As Double, ByVal t As Double, _
        ByVal u As Double)
        '形状関数の2階の偏微分
        's･･･要素のξ座標
        't･･･要素のη座標
        'u･･･要素のζ座標
        Dim s1, s1m, s2m, t1, t1m, t2m, u1, u1m, u2m As Double
        s1 = 1 + s
        s1m = 1 - s
        s2m = 1 - s * s
        t1 = 1 + t
        t1m = 1 - t
        t2m = 1 - t * t
        u1 = 1 + u
        u1m = 1 - u
        u2m = 1 - u * u

        Dss(1) = t1m * u1 / 4
        Dtt(1) = s1m * u1 / 4
        Duu(1) = s1m * t1m / 4
        Dst(1) = -u1 * (s + t - u + 1 + s) / 8 + t1m * u1 / 8
        Dtu(1) = s1m * (s + t - u + 1 + t) / 8 - s1m * u1 / 8
        Dus(1) = t1m * (s + t - u + 1 + s) / 8 - t1m * u1 / 8

        Dss(2) = -t1m * u1 / 2
        Dtt(2) = 0
        Duu(2) = 0
        Dst(2) = u1 * s / 2
        Dtu(2) = -s2m / 4
        Dus(2) = -t1m * s / 2

        Dss(3) = t1m * u1 / 4
        Dtt(3) = s1 * u1 / 4
        Duu(3) = s1 * t1m / 4
        Dst(3) = -u1 * (s - t + u - 1 + s) / 8 - t1m * u1 / 8
        Dtu(3) = s1 * (-s + t - u + 1 + t) / 8 - s1 * u1 / 8
        Dus(3) = t1m * (s - t + u - 1 + s) / 8 + t1m * u1 / 8

        Dss(4) = 0
        Dtt(4) = -s1 * u1 / 2
        Duu(4) = 0
        Dst(4) = -t * u1 / 2
        Dtu(4) = -s1 * t / 2
        Dus(4) = t2m / 4

        Dss(5) = t1 * u1 / 4
        Dtt(5) = s1 * u1 / 4
        Duu(5) = s1 * t1 / 4
        Dst(5) = u1 * (s + t + u - 1 + s) / 8 + t1 * u1 / 8
        Dtu(5) = s1 * (s + t + u - 1 + t) / 8 + s1 * u1 / 8
        Dus(5) = t1 * (s + t + u - 1 + s) / 8 + t1 * u1 / 8

        Dss(6) = -t1 * u1 / 2
        Dtt(6) = 0
        Duu(6) = 0
        Dst(6) = -u1 * s / 2
        Dtu(6) = s2m / 4
        Dus(6) = -t1 * s / 2

        Dss(7) = t1 * u1 / 4
        Dtt(7) = s1m * u1 / 4
        Duu(7) = s1m * t1 / 4
        Dst(7) = u1 * (s - t - u + 1 + s) / 8 - t1 * u1 / 8
        Dtu(7) = s1m * (-s + t + u - 1 + t) / 8 + s1m * u1 / 8
        Dus(7) = t1 * (s - t - u + 1 + s) / 8 - t1 * u1 / 8

        Dss(8) = 0
        Dtt(8) = -s1m * u1 / 2
        Duu(8) = 0
        Dst(8) = t * u1 / 2
        Dtu(8) = -s1m * t / 2
        Dus(8) = -t2m / 4

        Dss(9) = 0
        Dtt(9) = 0
        Duu(9) = -s1m * t1m / 2
        Dst(9) = u2m / 4
        Dtu(9) = s1m * u / 2
        Dus(9) = t1m * u / 2

        Dss(10) = 0
        Dtt(10) = 0
        Duu(10) = -s1 * t1m / 2
        Dst(10) = -u2m / 4
        Dtu(10) = s1 * u / 2
        Dus(10) = -t1m * u / 2

        Dss(11) = 0
        Dtt(11) = 0
        Duu(11) = -s1 * t1 / 2
        Dst(11) = u2m / 4
        Dtu(11) = -s1 * u / 2
        Dus(11) = -t1 * u / 2

        Dss(12) = 0
        Dtt(12) = 0
        Duu(12) = -s1m * t1 / 2
        Dst(12) = -u2m / 4
        Dtu(12) = -s1m * u / 2
        Dus(12) = t1 * u / 2

        Dss(13) = t1m * u1m / 4
        Dtt(13) = s1m * u1m / 4
        Duu(13) = s1m * t1m / 4
        Dst(13) = -u1m * (s + t + u + 1 + s) / 8 + t1m * u1m / 8
        Dtu(13) = -s1m * (s + t + u + 1 + t) / 8 + s1m * u1m / 8
        Dus(13) = -t1m * (s + t + u + 1 + s) / 8 + t1m * u1m / 8

        Dss(14) = -t1m * u1m / 2
        Dtt(14) = 0
        Duu(14) = 0
        Dst(14) = u1m * s / 2
        Dtu(14) = s2m / 4
        Dus(14) = t1m * s / 2

        Dss(15) = t1m * u1m / 4
        Dtt(15) = s1 * u1m / 4
        Duu(15) = s1 * t1m / 4
        Dst(15) = -u1m * (s - t - u - 1 + s) / 8 - t1m * u1m / 8
        Dtu(15) = -s1 * (-s + t + u + 1 + t) / 8 + s1 * u1m / 8
        Dus(15) = -t1m * (s - t - u - 1 + s) / 8 - t1m * u1m / 8

        Dss(16) = 0
        Dtt(16) = -s1 * u1m / 2
        Duu(16) = 0
        Dst(16) = -t * u1m / 2
        Dtu(16) = s1 * t / 2
        Dus(16) = -t2m / 4

        Dss(17) = t1 * u1m / 4
        Dtt(17) = s1 * u1m / 4
        Duu(17) = s1 * t1 / 4
        Dst(17) = u1m * (s + t - u - 1 + s) / 8 + t1 * u1m / 8
        Dtu(17) = -s1 * (s + t - u - 1 + t) / 8 - s1 * u1m / 8
        Dus(17) = -t1 * (s + t - u - 1 + s) / 8 - t1 * u1m / 8

        Dss(18) = -t1 * u1m / 2
        Dtt(18) = 0
        Duu(18) = 0
        Dst(18) = -u1m * s / 2
        Dtu(18) = -s2m / 4
        Dus(18) = t1 * s / 2

        Dss(19) = t1 * u1m / 4
        Dtt(19) = s1m * u1m / 4
        Duu(19) = s1m * t1 / 4
        Dst(19) = u1m * (s - t + u + 1 + s) / 8 - t1 * u1m / 8
        Dtu(19) = -s1m * (-s + t - u - 1 + t) / 8 - s1m * u1m / 8
        Dus(19) = -t1 * (s - t + u + 1 + s) / 8 + t1 * u1m / 8

        Dss(20) = 0
        Dtt(20) = -s1m * u1m / 2
        Duu(20) = 0
        Dst(20) = t * u1m / 2
        Dtu(20) = s1m * t / 2
        Dus(20) = t2m / 4

    End Sub

    Sub ShellSort2(ByVal kbuf() As Integer, ByRef kcbuf() As Integer, _
    ByVal xbuf() As Single, ByVal ybuf() As Single, ByVal zbuf() As Single)
        'ｼｪﾙｿｰﾄ
        Dim i As Integer
        Dim nsort(Npoint) As Integer
        Dim work(Npoint) As Single

        Select Case kbuf(1)
            Case 1
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = Xdata(i)
                Next i
            Case 2
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = Ydata(i)
                Next i
            Case 3
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = Zdata(i)
                Next i
        End Select

        Call ShellSortStd(Npoint, nsort, work)    'ｼｪｯﾙｿｰﾄ基本

        Call MiniSort2(kbuf, nsort, work, xbuf, ybuf, zbuf)    'ﾐﾆｿｰﾄ

        For i = 1 To Npoint
            kcbuf(nsort(i)) = i
        Next i

    End Sub

    Sub ShellSortStd(ByVal nx As Integer, ByRef nsort() As Integer, ByRef work() As Single)
        'ｼｪﾙｿｰﾄ基本
        'nx･･･変数の数
        'nsort()･･ｿｰﾄ順の元変数の番号
        'work()･･･ｿｰﾄ順の変数値
        Dim i, ib, j, m As Integer
        Dim varb As Single

        m = nx
        Do While m > 1
            m = m \ 2
            For j = m + 1 To nx
                varb = work(j)
                ib = nsort(j)
                i = j - m
                Do While work(i) >= varb
                    work(i + m) = work(i)
                    nsort(i + m) = nsort(i)
                    i = i - m
                    If i <= 0 Then
                        Exit Do
                    End If
                Loop
                work(i + m) = varb
                nsort(i + m) = ib
            Next j
        Loop

    End Sub

    Sub StressCheck(ByVal jmax As Integer, ByVal nmax As Integer, _
    ByVal sgb() As Double, ByRef spb() As Double)
        '要素節点応力ﾁｪｯｸ
        'jmax･･･積分点数
        'nmax･･･節点数
        'sgb()･･･ｶﾞｳｽ点応力
        'spb()･･･節点応力
        Dim i, j As Integer
        Dim b1, c1, c2 As Single
        Dim smax, smin As Single
        c1 = 1.2    'ｶｯﾄ上限
        c2 = 0.8    'ｶｯﾄ下限
        For j = 1 To jmax
            b1 = sgb(j)
            If j = 1 Then
                smax = b1
                smin = b1
            Else
                If b1 > smax Then
                    smax = b1
                ElseIf b1 < smin Then
                    smin = b1
                End If
            End If
        Next j
        b1 = smax
        If b1 >= 0 Then
            smax = c1 * b1
        Else
            smax = c2 * b1
        End If
        b1 = smin
        If b1 >= 0 Then
            smin = c2 * b1
        Else
            smin = c1 * b1
        End If

        For i = 1 To nmax
            If spb(i) > smax Then
                spb(i) = smax
            ElseIf spb(i) < smin Then
                spb(i) = smin
            End If
        Next i

    End Sub

    Sub SurfaceCond(ByVal kb As Integer, ByRef kind As Integer, _
        ByRef kbuf() As Integer, ByRef gpos() As Double)
        '境界条件用の面設定
        'kb･･･面番号
        'kind･･･面分類（1:面1と2、2:面3と4、3:面5と6）
        'kbuf()･･･面節点の基本番号
        'gpos()･･･積分点のξηζ座標
        Dim i As Integer
        Select Case kb
            Case 1
                kind = 1
                gpos(3) = 1
                For i = 1 To 8
                    kbuf(i) = i
                Next i
            Case 2
                kind = 1
                gpos(3) = -1
                kbuf(1) = 13
                kbuf(2) = 20
                kbuf(3) = 19
                kbuf(4) = 18
                kbuf(5) = 17
                kbuf(6) = 16
                kbuf(7) = 15
                kbuf(8) = 14
            Case 3
                kind = 2
                gpos(2) = -1
                kbuf(1) = 1
                kbuf(2) = 9
                kbuf(3) = 13
                kbuf(4) = 14
                kbuf(5) = 15
                kbuf(6) = 10
                kbuf(7) = 3
                kbuf(8) = 2
            Case 4
                kind = 2
                gpos(2) = 1
                kbuf(1) = 5
                kbuf(2) = 11
                kbuf(3) = 17
                kbuf(4) = 18
                kbuf(5) = 19
                kbuf(6) = 12
                kbuf(7) = 7
                kbuf(8) = 6
            Case 5
                kind = 3
                gpos(1) = 1
                kbuf(1) = 3
                kbuf(2) = 10
                kbuf(3) = 15
                kbuf(4) = 16
                kbuf(5) = 17
                kbuf(6) = 11
                kbuf(7) = 5
                kbuf(8) = 4
            Case 6
                kind = 3
                gpos(1) = -1
                kbuf(1) = 7
                kbuf(2) = 12
                kbuf(3) = 19
                kbuf(4) = 20
                kbuf(5) = 13
                kbuf(6) = 9
                kbuf(7) = 1
                kbuf(8) = 8
        End Select

    End Sub

    Sub SurfaceDraw(ByRef gr As Graphics, ByVal k1 As Integer,
        ByVal k2 As Integer, ByVal kind As Integer)
        '面の作画
        'k1,k2･･･要素の開始・終了番号
        'kind･･･面外周の作画有無（0:なし、1:有り）
        Dim aa$
        Dim ie, iex, j, jmax, k, kb, kbx, kk, kmax, nb, nb2, nmax As Integer
        Dim xb, xd, yb, yd As Single
        Dim fnt As Font = New Font("ＭＳ ゴシック", KnumSize)
        Dim fnt2 As Font = New Font("ＭＳ ゴシック", KnumSb, FontStyle.Bold)
        Dim kdisp(3) As Integer

        If KindElem = 0 Then
            jmax = 6
            kmax = 8
            nmax = 20
        End If
        If KdispENum > 0 Then
            kdisp(1) = 1
        Else
            kdisp(1) = 0
        End If
        If KdispPNum > 0 Then
            kdisp(2) = 1
        Else
            kdisp(2) = 0
        End If
        If KdispSNum > 0 Then
            kdisp(3) = 1
        Else
            kdisp(3) = 0
        End If
        If KmouseElem > 0 Then
            nb2 = NSdraw + 1
        Else
            nb2 = NSdraw
        End If
        For nb = 1 To nb2
            If nb <= NSdraw Then
                ie = KEorder(nb)
                kb = KSorder(nb)
                If ie = KmouseElem And kb = KmouseSurf Then
                    iex = ie
                    kbx = kb
                End If
            Else
                If iex = 0 Then
                    Exit For
                End If
                ie = iex
                kb = kbx
                kdisp(1) = 1
                kdisp(2) = 1
                kdisp(3) = 1
            End If
            If KindElem = 0 Then
                Call SurfaceSet(ie, kb, KsurfPoint) '面の設定
            End If
            For k = 1 To kmax
                kk = KsurfPoint(k)
                xd = Xscreen(kk)
                yd = Yscreen(kk)
                Call XYCalc(xd, yd)     'X,Yの作画座標計算
                Xdraw(k) = xd
                Ydraw(k) = yd
            Next k
            If nb <= NSdraw Then
                Call PaintExe(gr, kmax, Color.White, kind)
            End If
            If KnumDisp > 0 Then
                xb = 0 : yb = 0
                For k = 1 To kmax
                    xb = xb + Xdraw(k)
                    yb = yb + Ydraw(k)
                    If kdisp(2) > 0 Then
                        kk = KsurfPoint(k)
                        xd = Xdraw(k)
                        yd = Ydraw(k)
                        aa$ = LTrim$(Str$(kk))
                        gr.DrawString(aa$, fnt, Brushes.Black, xd, yd)
                    End If
                Next k
                If kdisp(3) > 0 Then
                    xd = xb / kmax
                    yd = yb / kmax
                    aa$ = "(" + LTrim$(Str$(kb)) + ")"
                    gr.DrawString(aa$, fnt, Brushes.Black, xd, yd)
                End If
                If kdisp(1) > 0 Then
                    xb = 0 : yb = 0
                    For j = 1 To nmax
                        kb = NNelem(ie, j)
                        xd = Xscreen(kb)
                        yd = Yscreen(kb)
                        Call XYCalc(xd, yd) 'X,Yの作画座標計算
                        xb = xb + xd
                        yb = yb + yd
                    Next j
                    xd = xb / nmax
                    yd = yb / nmax
                    aa$ = LTrim$(Str$(ie))
                    gr.DrawString(aa$, fnt2, Brushes.Black, xd, yd)
                End If
            End If
        Next nb

    End Sub

    Sub SurfaceSet(ByVal ib As Integer, ByVal kb As Integer, _
       ByRef kbuf() As Integer)
        '面の設定
        'ib･･･要素番号
        'kb･･･面番号
        'kbuf()･･･面の節点番号
        Select Case kb
            Case 1
                kbuf(1) = NNelem(ib, 1)
                kbuf(2) = NNelem(ib, 2)
                kbuf(3) = NNelem(ib, 3)
                kbuf(4) = NNelem(ib, 4)
                kbuf(5) = NNelem(ib, 5)
                kbuf(6) = NNelem(ib, 6)
                kbuf(7) = NNelem(ib, 7)
                kbuf(8) = NNelem(ib, 8)
            Case 2
                kbuf(1) = NNelem(ib, 13)
                kbuf(2) = NNelem(ib, 20)
                kbuf(3) = NNelem(ib, 19)
                kbuf(4) = NNelem(ib, 18)
                kbuf(5) = NNelem(ib, 17)
                kbuf(6) = NNelem(ib, 16)
                kbuf(7) = NNelem(ib, 15)
                kbuf(8) = NNelem(ib, 14)
            Case 3
                kbuf(1) = NNelem(ib, 1)
                kbuf(2) = NNelem(ib, 9)
                kbuf(3) = NNelem(ib, 13)
                kbuf(4) = NNelem(ib, 14)
                kbuf(5) = NNelem(ib, 15)
                kbuf(6) = NNelem(ib, 10)
                kbuf(7) = NNelem(ib, 3)
                kbuf(8) = NNelem(ib, 2)
            Case 4
                kbuf(1) = NNelem(ib, 5)
                kbuf(2) = NNelem(ib, 11)
                kbuf(3) = NNelem(ib, 17)
                kbuf(4) = NNelem(ib, 18)
                kbuf(5) = NNelem(ib, 19)
                kbuf(6) = NNelem(ib, 12)
                kbuf(7) = NNelem(ib, 7)
                kbuf(8) = NNelem(ib, 6)
            Case 5
                kbuf(1) = NNelem(ib, 3)
                kbuf(2) = NNelem(ib, 10)
                kbuf(3) = NNelem(ib, 15)
                kbuf(4) = NNelem(ib, 16)
                kbuf(5) = NNelem(ib, 17)
                kbuf(6) = NNelem(ib, 11)
                kbuf(7) = NNelem(ib, 5)
                kbuf(8) = NNelem(ib, 4)
            Case 6
                kbuf(1) = NNelem(ib, 7)
                kbuf(2) = NNelem(ib, 12)
                kbuf(3) = NNelem(ib, 19)
                kbuf(4) = NNelem(ib, 20)
                kbuf(5) = NNelem(ib, 13)
                kbuf(6) = NNelem(ib, 9)
                kbuf(7) = NNelem(ib, 1)
                kbuf(8) = NNelem(ib, 8)
        End Select

    End Sub

    Sub SurfaceVector(ByRef planevec() As Single, ByVal xbuf() As Single, _
        ByVal ybuf() As Single, ByVal zbuf() As Single)
        '面の法線ﾍﾞｸﾄﾙの計算
        'planevec()･･･面の法線ﾍﾞｸﾄﾙの各方向余弦
        'xbuf(),ybuf(),zbuf()･･･3点のX,Y,Z座標
        Dim b1, b2, b3, b4 As Single
        b1 = (ybuf(2) - ybuf(1)) * (zbuf(3) - zbuf(1))
        b1 = b1 - (zbuf(2) - zbuf(1)) * (ybuf(3) - ybuf(1))
        b2 = (zbuf(2) - zbuf(1)) * (xbuf(3) - xbuf(1))
        b2 = b2 - (xbuf(2) - xbuf(1)) * (zbuf(3) - zbuf(1))
        b3 = (xbuf(2) - xbuf(1)) * (ybuf(3) - ybuf(1))
        b3 = b3 - (ybuf(2) - ybuf(1)) * (xbuf(3) - xbuf(1))
        b4 = Math.Sqrt(b1 * b1 + b2 * b2 + b3 * b3)
        planevec(1) = b1 / b4
        planevec(2) = b2 / b4
        planevec(3) = b3 / b4

    End Sub

    Sub SurfPileCheck(ByVal nb As Integer)
        '面の重なりのﾁｪｯｸ
        'nb･･･面の数
        'KAppSide(,)が1の面が重なっている場合に遠方の面は0に変更
        Dim i, ib, ic, ie, ie2, ip, j, jmax, kb, kb2, kh, nd, nmax As Integer
        Dim b1, b2, epsA, epsL As Single
        Dim ksbuf(8) As Integer
        Dim nebuf(nb), nsbuf(nb) As Integer
        Dim xbuf(8), xbuf2(8), xrange(2), xrange2(2) As Single
        Dim ybuf(8), ybuf2(8), yrange(2), yrange2(2) As Single

        If KindElem = 0 Then
            jmax = 6
            nmax = 8
        End If

        nd = 0
        For i = 1 To NSdraw
            ie = KEorder(i)
            kb = KSorder(i)
            If KAppSide(ie, kb) > 0 Then
                nd = nd + 1
                nebuf(nd) = ie
                nsbuf(nd) = kb
            End If
        Next i

        For i = 1 To nd
            ie = nebuf(i)
            If ie > 0 Then
                kb = nsbuf(i)
                If KindElem = 0 Then
                    Call SurfaceSet(ie, kb, KsurfPoint)     '面の設定
                End If
                For ip = 1 To nmax
                    ib = KsurfPoint(ip)
                    xbuf(ip) = Xscreen(ib)
                    ybuf(ip) = Yscreen(ib)
                Next ip
                Call AxisCheckSet(nmax, xrange, yrange, xbuf, ybuf)
                'X･Y座標の最小・最大値のﾁｪｯｸ（ｾｯﾄ）

                b1 = xrange(2) - xrange(1)
                b2 = yrange(2) - yrange(1)
                If b2 > b1 Then
                    b1 = b2
                End If
                epsL = 0.001 * b1  '微小誤差長さ
                epsA = epsL * epsL  '微小誤差面積
                For j = i + 1 To nd
                    ie2 = nebuf(j)
                    If ie2 > 0 Then
                        kb2 = nsbuf(j)
                        If KindElem = 0 Then
                            Call SurfaceSet(ie2, kb2, ksbuf)     '面の設定
                        End If
                        For ip = 1 To nmax
                            ib = ksbuf(ip)
                            xbuf2(ip) = Xscreen(ib)
                            ybuf2(ip) = Yscreen(ib)
                        Next ip
                        Call AxisCheckSet(nmax, xrange2, yrange2, xbuf2, ybuf2)
                        'X･Y座標の最小・最大値のﾁｪｯｸ（ｾｯﾄ）

                        Call RangeCheck(kh, epsL, xrange, xrange2, yrange, yrange2)
                        '範囲ﾁｪｯｸ

                        If kh = 0 Then
                            Call AreaCheck(kh, nmax, epsA, xbuf, xbuf2, ybuf, ybuf2)
                            '2面の重なりﾁｪｯｸ

                            If kh > 0 Then
                                Call DistComp(ic, ksbuf)    '距離比較
                                If ic = 0 Then
                                    KAppSide(ie2, nsbuf(j)) = 0
                                    nebuf(j) = 0
                                Else
                                    KAppSide(ie, nsbuf(i)) = 0
                                    nebuf(i) = 0
                                    Exit For
                                End If
                            End If
                        End If
                    End If
                Next j
            End If
        Next i

    End Sub

    Sub SurfSort(ByVal k1 As Integer, ByVal k2 As Integer)
        '作画面のｿｰﾃｨﾝｸﾞ
        'k1,k2･･･開始・終了要素番号
        Dim i1, i2, ic, ie, ii, ik1, ik2, j, k, kmax, nmax, nb As Integer
        Dim kbuf(8) As Integer

        If KindElem = 0 Then
            kmax = 6
            nmax = 8
        End If
        NSdraw = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If KSelem(ie, k) = 0 Then
                        NSdraw = NSdraw + 1
                    End If
                Next k
            End If
        Next ie
        ReDim KEorder(NSdraw), KSorder(NSdraw)
        nb = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If KSelem(ie, k) = 0 Then
                        nb = nb + 1
                        KEorder(nb) = ie
                        KSorder(nb) = k
                    End If
                Next k
            End If
        Next ie

        For ii = 1 To NSdraw - 1
            i1 = KEorder(ii)
            ik1 = KSorder(ii)
            If KindElem = 0 Then
                Call SurfaceSet(i1, ik1, KsurfPoint)    '面の設定
            End If

            For j = ii + 1 To NSdraw
                i2 = KEorder(j)
                ik2 = KSorder(j)
                If KindElem = 0 Then
                    Call SurfaceSet(i2, ik2, kbuf)      '面の設定
                End If

                Call DistComp(ic, kbuf)    '距離比較
                If ic = 0 Then
                    KEorder(ii) = i2
                    KSorder(ii) = ik2
                    KEorder(j) = i1
                    KSorder(j) = ik1
                    i1 = i2
                    ik1 = ik2
                    For k = 1 To nmax
                        KsurfPoint(k) = kbuf(k)
                    Next k
                End If
            Next j
        Next ii

    End Sub

    Sub TDatDupCheck(ByRef nd As Integer)
        '温度固定節点の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, nb As Integer
        Dim kbuf(NBtempP) As Integer

        nd = 0
        For i = 1 To NBtempP - 1
            If kbuf(i) = 0 Then
                For ii = i + 1 To NBtempP
                    If NTrestP(i) = NTrestP(ii) Then
                        nd = nd + 1
                        kbuf(ii) = -1
                        TempFixP(i) = TempFixP(ii)
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(NBtempP) As Integer
            Dim buf2(NBtempP) As Single
            For i = 1 To NBtempP
                nbuf1(i) = NTrestP(i)
                buf2(i) = TempFixP(i)
            Next i
            nb = NBtempP - nd
            ReDim NTrestP(nb), TempFixP(nb)
            nb = 0
            For i = 1 To NBtempP
                If kbuf(i) = 0 Then
                    nb = nb + 1
                    NTrestP(nb) = nbuf1(i)
                    TempFixP(nb) = buf2(i)
                End If
            Next i
            NBtempP = nb
            NBtempPb = nb
        End If

    End Sub

    Sub TimCal(ByRef ktim As Integer, ByRef timn$)
        'ﾀｲﾏｰ計算
        'ktim･･･時間継続指定（0:時間ｸﾘｱする、1:時間継続する）
        'timn$･･･時分秒に換算した経過時間
        Dim rtime As Date = Now
        Dim rhour, rmin, sec As Single
        If ktim > 0 Then
            sec = DateDiff(DateInterval.Second, Timst, rtime)
            rhour = Int(sec / 3600) : sec = sec - rhour * 3600
            rmin = Int(sec / 60) : sec = Int(sec - rmin * 60)
            timn$ = Str$(rhour) + "h" + Str$(rmin) + "min" + Str$(sec) + "s"
        Else
            Timst = rtime
        End If
    End Sub

    Sub TmatMulti(ByRef tr1buf(,) As Single, ByRef tr2buf(,) As Single)
        '変換行列の掛算
        'tr1buf(,)･･･最初の変換行列
        'tr2buf(,)･･･次の変換行列および結果の変化行列
        Dim i, j, k As Integer
        Dim b1 As Single
        Dim buf(4, 4) As Single

        For i = 1 To 4
            For j = 1 To 4
                buf(i, j) = tr2buf(i, j)
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf(i, k) * tr1buf(k, j)
                Next k
                tr2buf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalA(ByVal xs As Single, ByVal ys As Single, _
        ByVal zs As Single, ByVal xr As Single, ByVal yr As Single, _
        ByVal zr As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        '先にZ,Y,X軸回りの回転、最後に座標軸方向の平行移動
        'xs,ys,zs･･･X,Y,Z方向移動量
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single
        ReDim TransMat(4, 4)

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i
        buf1(1, 4) = xs
        buf1(2, 4) = ys
        buf1(3, 4) = zs

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = b2
        buf3(2, 2) = 1
        buf3(3, 1) = -b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = -b2
        buf4(2, 1) = b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + TransMat(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalD(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
    ByRef trbuf(,) As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = b2
        buf3(2, 2) = 1
        buf3(3, 1) = -b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = -b2
        buf4(2, 1) = b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalE(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
    ByRef trbuf(,) As Single)
        '座標軸の合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = b2
        buf2(3, 2) = -b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = -b2
        buf3(2, 2) = 1
        buf3(3, 1) = b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = b2
        buf4(2, 1) = -b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmpFileIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Flow3DLiq.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            Fm$ = sr.ReadLine
            Fy$ = sr.ReadLine
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub TmpFileOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\Flow3DLiq.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(Fm$)
        sw.WriteLine(Fy$)
        sw.Close()

    End Sub

    Sub TrSet(ByRef tr(,) As Single)
        '節点応力計算のTr行列の設定
        Dim i, j As Integer
        Dim abuf As Single

        For i = 1 To 2
            For j = 1 To 27
                If i = 1 Then
                    Select Case j
                        Case 1
                            abuf = -0.1255884
                            tr(1, 1) = abuf : tr(1, 9) = abuf : tr(1, 21) = abuf
                            tr(3, 3) = abuf : tr(3, 19) = abuf : tr(3, 27) = abuf
                            tr(5, 9) = abuf : tr(5, 21) = abuf : tr(5, 25) = abuf
                            tr(7, 3) = abuf : tr(7, 7) = abuf : tr(7, 27) = abuf
                            tr(13, 3) = abuf : tr(13, 7) = abuf : tr(13, 19) = abuf
                            tr(15, 1) = abuf : tr(15, 21) = abuf : tr(15, 25) = abuf
                            tr(17, 7) = abuf : tr(17, 19) = abuf : tr(17, 27) = abuf
                            tr(19, 1) = abuf : tr(19, 9) = abuf : tr(19, 25) = abuf
                        Case 2
                            abuf = -0.3124648
                            tr(1, 2) = abuf : tr(1, 6) = abuf : tr(1, 12) = abuf
                            tr(3, 12) = abuf : tr(3, 20) = abuf : tr(3, 24) = abuf
                            tr(5, 18) = abuf : tr(5, 24) = abuf : tr(5, 26) = abuf
                            tr(7, 6) = abuf : tr(7, 8) = abuf : tr(7, 18) = abuf
                            tr(13, 2) = abuf : tr(13, 4) = abuf : tr(13, 10) = abuf
                            tr(15, 10) = abuf : tr(15, 20) = abuf : tr(15, 22) = abuf
                            tr(17, 16) = abuf : tr(17, 22) = abuf : tr(17, 26) = abuf
                            tr(19, 4) = abuf : tr(19, 8) = abuf : tr(19, 16) = abuf
                        Case 3
                            abuf = 2.374992
                            tr(1, 3) = abuf : tr(3, 21) = abuf
                            tr(5, 27) = abuf : tr(7, 9) = abuf
                            tr(13, 1) = abuf : tr(15, 19) = abuf
                            tr(17, 25) = abuf : tr(19, 7) = abuf
                        Case 4
                            abuf = 0.3148157
                            tr(1, 4) = abuf : tr(1, 8) = abuf : tr(1, 10) = abuf
                            tr(1, 18) = abuf : tr(1, 20) = abuf : tr(1, 24) = abuf
                            tr(3, 2) = abuf : tr(3, 6) = abuf : tr(3, 10) = abuf
                            tr(3, 18) = abuf : tr(3, 22) = abuf : tr(3, 26) = abuf
                            tr(5, 6) = abuf : tr(5, 8) = abuf : tr(5, 12) = abuf
                            tr(5, 16) = abuf : tr(5, 20) = abuf : tr(5, 22) = abuf
                            tr(7, 2) = abuf : tr(7, 4) = abuf : tr(7, 12) = abuf
                            tr(7, 16) = abuf : tr(7, 24) = abuf : tr(7, 26) = abuf
                            tr(13, 6) = abuf : tr(13, 8) = abuf : tr(13, 12) = abuf
                            tr(13, 16) = abuf : tr(13, 20) = abuf : tr(13, 22) = abuf
                            tr(15, 2) = abuf : tr(15, 4) = abuf : tr(15, 12) = abuf
                            tr(15, 16) = abuf : tr(15, 24) = abuf : tr(15, 26) = abuf
                            tr(17, 4) = abuf : tr(17, 8) = abuf : tr(17, 10) = abuf
                            tr(17, 18) = abuf : tr(17, 20) = abuf : tr(17, 24) = abuf
                            tr(19, 2) = abuf : tr(19, 6) = abuf : tr(19, 10) = abuf
                            tr(19, 18) = abuf : tr(19, 22) = abuf : tr(19, 26) = abuf
                        Case 5
                            abuf = -0.4882397
                            tr(1, 5) = abuf : tr(1, 11) = abuf : tr(1, 15) = abuf
                            tr(3, 11) = abuf : tr(3, 15) = abuf : tr(3, 23) = abuf
                            tr(5, 15) = abuf : tr(5, 17) = abuf : tr(5, 23) = abuf
                            tr(7, 5) = abuf : tr(7, 15) = abuf : tr(7, 17) = abuf
                            tr(13, 5) = abuf : tr(13, 11) = abuf : tr(13, 13) = abuf
                            tr(15, 11) = abuf : tr(15, 13) = abuf : tr(15, 23) = abuf
                            tr(17, 13) = abuf : tr(17, 17) = abuf : tr(17, 23) = abuf
                            tr(19, 5) = abuf : tr(19, 13) = abuf : tr(19, 17) = abuf
                        Case 7
                            abuf = -0.1614496
                            tr(1, 7) = abuf : tr(1, 19) = abuf : tr(1, 27) = abuf
                            tr(3, 1) = abuf : tr(3, 9) = abuf : tr(3, 25) = abuf
                            tr(5, 3) = abuf : tr(5, 7) = abuf : tr(5, 19) = abuf
                            tr(7, 1) = abuf : tr(7, 21) = abuf : tr(7, 25) = abuf
                            tr(13, 9) = abuf : tr(13, 21) = abuf : tr(13, 25) = abuf
                            tr(15, 3) = abuf : tr(15, 7) = abuf : tr(15, 27) = abuf
                            tr(17, 1) = abuf : tr(17, 9) = abuf : tr(17, 21) = abuf
                            tr(19, 3) = abuf : tr(19, 19) = abuf : tr(19, 27) = abuf
                        Case 13
                            abuf = 0.2289803
                            tr(1, 13) = abuf : tr(1, 17) = abuf : tr(1, 23) = abuf
                            tr(3, 5) = abuf : tr(3, 13) = abuf : tr(3, 17) = abuf
                            tr(5, 5) = abuf : tr(5, 11) = abuf : tr(5, 13) = abuf
                            tr(7, 11) = abuf : tr(7, 13) = abuf : tr(7, 23) = abuf
                            tr(13, 15) = abuf : tr(13, 17) = abuf : tr(13, 23) = abuf
                            tr(15, 5) = abuf : tr(15, 15) = abuf : tr(15, 17) = abuf
                            tr(17, 5) = abuf : tr(17, 11) = abuf : tr(17, 15) = abuf
                            tr(19, 11) = abuf : tr(19, 15) = abuf : tr(19, 23) = abuf
                        Case 14
                            abuf = -0.2962959
                            tr(1, 14) = abuf : tr(3, 14) = abuf
                            tr(5, 14) = abuf : tr(7, 14) = abuf
                            tr(13, 14) = abuf : tr(15, 14) = abuf
                            tr(17, 14) = abuf : tr(19, 14) = abuf
                        Case 16
                            abuf = -0.16902
                            tr(1, 16) = abuf : tr(1, 22) = abuf : tr(1, 26) = abuf
                            tr(3, 4) = abuf : tr(3, 8) = abuf : tr(3, 16) = abuf
                            tr(5, 2) = abuf : tr(5, 4) = abuf : tr(5, 10) = abuf
                            tr(7, 10) = abuf : tr(7, 20) = abuf : tr(7, 22) = abuf
                            tr(13, 18) = abuf : tr(13, 24) = abuf : tr(13, 26) = abuf
                            tr(15, 6) = abuf : tr(15, 8) = abuf : tr(15, 18) = abuf
                            tr(17, 2) = abuf : tr(17, 6) = abuf : tr(17, 12) = abuf
                            tr(19, 12) = abuf : tr(19, 20) = abuf : tr(19, 24) = abuf
                        Case 25
                            abuf = 0.1157502
                            tr(1, 25) = abuf : tr(3, 7) = abuf
                            tr(5, 1) = abuf : tr(7, 19) = abuf
                            tr(13, 27) = abuf : tr(15, 9) = abuf
                            tr(17, 3) = abuf : tr(19, 21) = abuf
                    End Select
                Else
                    Select Case j
                        Case 1
                            abuf = 0.1111112
                            tr(2, 1) = abuf : tr(2, 9) = abuf
                            tr(2, 19) = abuf : tr(2, 27) = abuf
                            tr(4, 3) = abuf : tr(4, 9) = abuf
                            tr(4, 19) = abuf : tr(4, 25) = abuf
                            tr(6, 3) = abuf : tr(6, 21) = abuf
                            tr(6, 7) = abuf : tr(6, 25) = abuf
                            tr(8, 1) = abuf : tr(8, 7) = abuf
                            tr(8, 21) = abuf : tr(8, 27) = abuf
                            tr(9, 7) = abuf : tr(9, 9) = abuf
                            tr(9, 19) = abuf : tr(9, 21) = abuf
                            tr(10, 1) = abuf : tr(10, 3) = abuf
                            tr(10, 25) = abuf : tr(10, 27) = abuf
                            tr(11, 7) = abuf : tr(11, 9) = abuf
                            tr(11, 19) = abuf : tr(11, 21) = abuf
                            tr(12, 1) = abuf : tr(12, 3) = abuf
                            tr(12, 25) = abuf : tr(12, 27) = abuf
                            tr(14, 3) = abuf : tr(14, 21) = abuf
                            tr(14, 7) = abuf : tr(14, 25) = abuf
                            tr(16, 1) = abuf : tr(16, 7) = abuf
                            tr(16, 21) = abuf : tr(16, 27) = abuf
                            tr(18, 1) = abuf : tr(18, 19) = abuf
                            tr(18, 9) = abuf : tr(18, 27) = abuf
                            tr(20, 3) = abuf : tr(20, 9) = abuf
                            tr(20, 19) = abuf : tr(20, 25) = abuf
                        Case 2
                            abuf = -0.2707217
                            tr(2, 2) = abuf : tr(2, 20) = abuf
                            tr(2, 6) = abuf : tr(2, 24) = abuf
                            tr(4, 12) = abuf : tr(4, 18) = abuf
                            tr(4, 20) = abuf : tr(4, 26) = abuf
                            tr(6, 6) = abuf : tr(6, 24) = abuf
                            tr(6, 8) = abuf : tr(6, 26) = abuf
                            tr(8, 2) = abuf : tr(8, 8) = abuf
                            tr(8, 12) = abuf : tr(8, 18) = abuf
                            tr(9, 4) = abuf : tr(9, 6) = abuf
                            tr(9, 10) = abuf : tr(9, 12) = abuf
                            tr(10, 10) = abuf : tr(10, 12) = abuf
                            tr(10, 22) = abuf : tr(10, 24) = abuf
                            tr(11, 16) = abuf : tr(11, 18) = abuf
                            tr(11, 22) = abuf : tr(11, 24) = abuf
                            tr(12, 4) = abuf : tr(12, 6) = abuf
                            tr(12, 16) = abuf : tr(12, 18) = abuf
                            tr(14, 2) = abuf : tr(14, 20) = abuf
                            tr(14, 4) = abuf : tr(14, 22) = abuf
                            tr(16, 10) = abuf : tr(16, 16) = abuf
                            tr(16, 20) = abuf : tr(16, 26) = abuf
                            tr(18, 4) = abuf : tr(18, 22) = abuf
                            tr(18, 8) = abuf : tr(18, 26) = abuf
                            tr(20, 2) = abuf : tr(20, 8) = abuf
                            tr(20, 10) = abuf : tr(20, 16) = abuf
                        Case 3
                            abuf = 0.3262776
                            tr(2, 3) = abuf : tr(2, 21) = abuf
                            tr(4, 21) = abuf : tr(4, 27) = abuf
                            tr(6, 9) = abuf : tr(6, 27) = abuf
                            tr(8, 3) = abuf : tr(8, 9) = abuf
                            tr(9, 1) = abuf : tr(9, 3) = abuf
                            tr(10, 19) = abuf : tr(10, 21) = abuf
                            tr(11, 25) = abuf : tr(11, 27) = abuf
                            tr(12, 7) = abuf : tr(12, 9) = abuf
                            tr(14, 1) = abuf : tr(14, 19) = abuf
                            tr(16, 19) = abuf : tr(16, 25) = abuf
                            tr(18, 7) = abuf : tr(18, 25) = abuf
                            tr(20, 1) = abuf : tr(20, 7) = abuf
                        Case 4
                            abuf = 0.1596107
                            tr(2, 4) = abuf : tr(2, 22) = abuf
                            tr(2, 8) = abuf : tr(2, 26) = abuf
                            tr(4, 2) = abuf : tr(4, 8) = abuf
                            tr(4, 10) = abuf : tr(4, 16) = abuf
                            tr(6, 2) = abuf : tr(6, 20) = abuf
                            tr(6, 4) = abuf : tr(6, 22) = abuf
                            tr(8, 10) = abuf : tr(8, 16) = abuf
                            tr(8, 20) = abuf : tr(8, 26) = abuf
                            tr(9, 16) = abuf : tr(9, 18) = abuf
                            tr(9, 22) = abuf : tr(9, 24) = abuf
                            tr(10, 4) = abuf : tr(10, 6) = abuf
                            tr(10, 16) = abuf : tr(10, 18) = abuf
                            tr(11, 4) = abuf : tr(11, 6) = abuf
                            tr(11, 10) = abuf : tr(11, 12) = abuf
                            tr(12, 10) = abuf : tr(12, 12) = abuf
                            tr(12, 22) = abuf : tr(12, 24) = abuf
                            tr(14, 6) = abuf : tr(14, 24) = abuf
                            tr(14, 8) = abuf : tr(14, 26) = abuf
                            tr(16, 2) = abuf : tr(16, 8) = abuf
                            tr(16, 12) = abuf : tr(16, 18) = abuf
                            tr(18, 2) = abuf : tr(18, 20) = abuf
                            tr(18, 6) = abuf : tr(18, 24) = abuf
                            tr(20, 12) = abuf : tr(20, 18) = abuf
                            tr(20, 20) = abuf : tr(20, 26) = abuf
                        Case 5
                            abuf = -0.2222226
                            tr(2, 5) = abuf : tr(2, 23) = abuf
                            tr(4, 11) = abuf : tr(4, 17) = abuf
                            tr(6, 5) = abuf : tr(6, 23) = abuf
                            tr(8, 11) = abuf : tr(8, 17) = abuf
                            tr(9, 13) = abuf : tr(9, 15) = abuf
                            tr(10, 13) = abuf : tr(10, 15) = abuf
                            tr(11, 13) = abuf : tr(11, 15) = abuf
                            tr(12, 13) = abuf : tr(12, 15) = abuf
                            tr(14, 5) = abuf : tr(14, 23) = abuf
                            tr(16, 11) = abuf : tr(16, 17) = abuf
                            tr(18, 5) = abuf : tr(18, 23) = abuf
                            tr(20, 11) = abuf : tr(20, 17) = abuf
                        Case 7
                            abuf = -0.1040551
                            tr(2, 7) = abuf : tr(2, 25) = abuf
                            tr(4, 1) = abuf : tr(4, 7) = abuf
                            tr(6, 1) = abuf : tr(6, 19) = abuf
                            tr(8, 19) = abuf : tr(8, 25) = abuf
                            tr(9, 25) = abuf : tr(9, 27) = abuf
                            tr(10, 7) = abuf : tr(10, 9) = abuf
                            tr(11, 1) = abuf : tr(11, 3) = abuf
                            tr(12, 19) = abuf : tr(12, 21) = abuf
                            tr(14, 9) = abuf : tr(14, 27) = abuf
                            tr(16, 3) = abuf : tr(16, 9) = abuf
                            tr(18, 3) = abuf : tr(18, 21) = abuf
                            tr(20, 21) = abuf : tr(20, 27) = abuf
                        Case 10
                            abuf = -0.194445
                            tr(2, 10) = abuf : tr(2, 18) = abuf
                            tr(4, 6) = abuf : tr(4, 22) = abuf
                            tr(6, 12) = abuf : tr(6, 16) = abuf
                            tr(8, 4) = abuf : tr(8, 24) = abuf
                            tr(9, 8) = abuf : tr(9, 20) = abuf
                            tr(10, 2) = abuf : tr(10, 26) = abuf
                            tr(11, 8) = abuf : tr(11, 20) = abuf
                            tr(12, 2) = abuf : tr(12, 26) = abuf
                            tr(14, 12) = abuf : tr(14, 16) = abuf
                            tr(16, 4) = abuf : tr(16, 24) = abuf
                            tr(18, 10) = abuf : tr(18, 18) = abuf
                            tr(20, 6) = abuf : tr(20, 22) = abuf
                        Case 11
                            abuf = 0.05555563
                            tr(2, 11) = abuf : tr(2, 15) = abuf
                            tr(4, 15) = abuf : tr(4, 23) = abuf
                            tr(6, 15) = abuf : tr(6, 17) = abuf
                            tr(8, 5) = abuf : tr(8, 15) = abuf
                            tr(9, 5) = abuf : tr(9, 11) = abuf
                            tr(10, 11) = abuf : tr(10, 23) = abuf
                            tr(11, 17) = abuf : tr(11, 23) = abuf
                            tr(12, 5) = abuf : tr(12, 17) = abuf
                            tr(14, 11) = abuf : tr(14, 13) = abuf
                            tr(16, 13) = abuf : tr(16, 23) = abuf
                            tr(18, 13) = abuf : tr(18, 17) = abuf
                            tr(20, 5) = abuf : tr(20, 13) = abuf
                        Case 12
                            abuf = 1.284386
                            tr(2, 12) = abuf : tr(4, 24) = abuf
                            tr(6, 18) = abuf : tr(8, 6) = abuf
                            tr(9, 2) = abuf : tr(10, 20) = abuf
                            tr(11, 26) = abuf : tr(12, 8) = abuf
                            tr(14, 10) = abuf : tr(16, 22) = abuf
                            tr(18, 16) = abuf : tr(20, 4) = abuf
                        Case 13
                            abuf = 0.05555541
                            tr(2, 13) = abuf : tr(2, 17) = abuf
                            tr(4, 5) = abuf : tr(4, 13) = abuf
                            tr(6, 11) = abuf : tr(6, 13) = abuf
                            tr(8, 13) = abuf : tr(8, 23) = abuf
                            tr(9, 17) = abuf : tr(9, 23) = abuf
                            tr(10, 5) = abuf : tr(10, 17) = abuf
                            tr(11, 5) = abuf : tr(11, 11) = abuf
                            tr(12, 11) = abuf : tr(12, 23) = abuf
                            tr(14, 15) = abuf : tr(14, 17) = abuf
                            tr(16, 5) = abuf : tr(16, 15) = abuf
                            tr(18, 11) = abuf : tr(18, 15) = abuf
                            tr(20, 15) = abuf : tr(20, 23) = abuf
                        Case 14
                            abuf = -0.111111
                            tr(2, 14) = abuf : tr(4, 14) = abuf
                            tr(6, 14) = abuf : tr(8, 14) = abuf
                            tr(9, 14) = abuf : tr(10, 14) = abuf
                            tr(11, 14) = abuf : tr(12, 14) = abuf
                            tr(14, 14) = abuf : tr(16, 14) = abuf
                            tr(18, 14) = abuf : tr(20, 14) = abuf
                        Case 16
                            abuf = -0.006608076
                            tr(2, 16) = abuf : tr(4, 4) = abuf
                            tr(6, 10) = abuf : tr(8, 22) = abuf
                            tr(9, 26) = abuf : tr(10, 8) = abuf
                            tr(11, 2) = abuf : tr(12, 20) = abuf
                            tr(14, 18) = abuf : tr(16, 6) = abuf
                            tr(18, 12) = abuf : tr(20, 24) = abuf
                    End Select
                End If
            Next j
        Next i

    End Sub

    Sub TSDatDupCheck(ByRef nd As Integer)
        '温度固定面の重複ﾁｪｯｸ・修正
        'nd･･･重複の数
        Dim i, ii, j, nb As Integer
        Dim ksbuf(NBtempS) As Integer

        nd = 0
        For i = 1 To NBtempS - 1
            If ksbuf(i) = 0 Then
                For ii = i + 1 To NBtempS
                    If KsurfTrest(i, 1) = KsurfTrest(ii, 1) Then
                        If KsurfTrest(i, 2) = KsurfTrest(ii, 2) Then
                            nd = nd + 1
                            ksbuf(ii) = -1
                            TempFixS(i) = TempFixS(ii)
                        End If
                    End If
                Next ii
            End If
        Next i

        If nd > 0 Then
            Dim nbuf1(NBtempS, 2) As Integer
            Dim buf2(NBtempS) As Single
            For i = 1 To NBtempS
                For j = 1 To 2
                    nbuf1(i, j) = KsurfTrest(i, j)
                Next j
                buf2(i) = TempFixS(i)
            Next i
            nb = NBtempS - nd
            ReDim KsurfTrest(nb, 2), TempFixS(nb)
            nb = 0
            For i = 1 To NBtempS
                If ksbuf(i) = 0 Then
                    nb = nb + 1
                    KsurfTrest(nb, 1) = nbuf1(i, 1)
                    KsurfTrest(nb, 2) = nbuf1(i, 2)
                    TempFixS(nb) = buf2(i)
                End If
            Next i
            NBtempS = nb
            NBtempSb = nb
        End If

    End Sub

    Sub UFileAddA()
        'UFileAの追加処理
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFileA(i) = UFileA(0) Then
                For j = i - 1 To 0 Step -1
                    UFileA(j + 1) = UFileA(j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFileA(i + 1) = UFileA(i)
            Next i
        End If
        UseFileAOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UseFileAIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\FL3DUFileA.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            i = 0
            Do Until sr.Peek = -1 'ﾌｧｲﾙ末尾まで
                i = i + 1
                UFileA(i) = sr.ReadLine
            Loop
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileAOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i As Integer

        ftmp$ = DirOpen$ + "\FL3DUFileA.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For i = 1 To 6
            If UFileA(i) <> "" Then
                sw.WriteLine(UFileA(i))
            Else
                Exit For
            End If
        Next i
        sw.Close()

    End Sub

    Sub VecMaxCalc(ByVal k1 As Integer, ByVal k2 As Integer, ByRef vecmax As Single)
        '速度ﾍﾞｸﾄﾙ最大値の計算
        'k1･･･作画初めの要素番号
        'k2･･･作画終りの要素番号
        'vecmax･･･速度ﾍﾞｸﾄﾙの最大値
        Dim ie, ii, ip, j, k, nmax As Integer
        Dim b1 As Single
        If KindElem = 0 Then
            nmax = 20
        End If
        vecmax = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For k = 1 To nmax
                    ip = NNelem(ie, k)
                    ii = 3 * (ip - 1)
                    b1 = 0
                    For j = 1 To 3
                        b1 = b1 + Tfp(ii + j) ^ 2
                    Next j
                    If b1 > vecmax Then
                        vecmax = b1
                    End If
                Next k
            End If
        Next ie
        vecmax = Math.Sqrt(vecmax)

    End Sub

    Function VelMset(ByRef tfb() As Double) As Double
        '最大速度算出
        Dim i, kk, kx, m As Integer
        Dim b1, velm As Double

        velm = 0
        For i = 1 To Npoint
            kx = Ndim * (Kcomp(i) - 1) + 1
            For m = 0 To 2
                kk = Lcomp(kx + m)
                If kk > 0 Then
                    b1 = Math.Abs(tfb(kk))
                    If b1 > velm Then
                        velm = b1
                    End If
                End If
            Next m
        Next i
        Return velm

    End Function

    Sub ViscEqua(ByRef et As Double, ByVal gd As Double,
        ByVal temp As Double)
        '粘度式計算
        'et･･･粘度
        'gd･･･剪断速度
        'temp･･･温度
        Dim b1, b2, gdb As Double
        gdb = gd
        If gdb < 1 Then
            gdb = 1
        End If
        If Kvisc > 0 Then
            Select Case Kvisc
                Case 1
                    b1 = Avisc(0) * Math.Exp(Avisc(1) * temp)
                    et = b1 * (gdb ^ Avisc(2))
                Case 2
                    b1 = Math.Log(gdb) / Math.Log(10)
                    b2 = Avisc(0) + Avisc(1) * b1 + Avisc(2) * temp
                    b2 = b2 + Avisc(3) * b1 * b1
                    b2 = b2 + Avisc(4) * b1 * temp + Avisc(5) * temp * temp
                    et = 10 ^ b2
                Case 3
                    b1 = Math.Log(gdb)
                    b2 = Avisc(0) + Avisc(1) * b1 + Avisc(2) * temp
                    b2 = b2 + Avisc(3) * b1 * b1
                    b2 = b2 + Avisc(4) * b1 * temp + Avisc(5) * temp * temp
                    et = Math.Exp(b2)
                Case 4
                    et = Avisc(0) + Avisc(1) * temp + Avisc(2) * temp * temp
            End Select
        End If

    End Sub

    Sub VolumeCalc()
        '要素体積の計算
        Dim ie As Integer
        Dim vbuf As Single
        ReDim VolE(Nelem)

        For ie = 1 To Nelem
            If KindElem = 0 Then
                Call ElemCalc(ie, vbuf)   '六面体要素の計算
            End If
            If vbuf <= 0 Then
                Dim aa$
                aa$ = "要素番号" + LTrim(Str(ie)) + "で、体積計算が負になりました！"
                MsgBox(aa$, 0, "データエラー")
                End
            End If
            VolE(ie) = vbuf
        Next ie

    End Sub

    Sub XYCalc(ByRef xb As Single, ByRef yb As Single)
        'X,Yの作画座標計算
        'xb･･･X座標値
        'yb･･･Y座標値
        Dim b1, b2 As Single
        b1 = xb - XaxisMin
        b2 = yb - YaxisMin
        If Cfix < 1 Then
            xb = b1 * XYscale + Xlower
            yb = Yupper - b2 * XYscale * Cfix
        Else
            xb = b1 * XYscale / Cfix + Xlower
            yb = Yupper - b2 * XYscale
        End If
    End Sub

End Module
